<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://xfinitysoft.com/
 * @since      5.0.0
 *
 * @package    Wc_Order_Limit
 * @subpackage Wc_Order_Limit/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wc_Order_Limit
 * @subpackage Wc_Order_Limit/admin
 * @author     xfinitysoft <support@xfinitysoft.com>
 */
class Wc_Order_Limit_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    5.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    5.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    5.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    5.0.0
	 */
	public function enqueue_styles() {
         global $post;
        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Wc_Order_Limit_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Wc_Order_Limit_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
          // Registering Admin Styles
        wp_register_style('bootstrap-css' , plugin_dir_url( __FILE__ ).'css/bootstrap.min.css',array());
        if( (isset($_GET['page']) && $_GET['page'] == 'wcol_settings') || (isset($_GET['page']) && $_GET['page'] == 'wcol-support')){
            wp_enqueue_style('bootstrap-css');
        }
        wp_enqueue_style('select2-css',plugin_dir_url( __FILE__ ).'css/select2.min.css', array());
        wp_enqueue_style('jquery-rain-date-time-css', plugin_dir_url( __FILE__ ).'css/jquery-rain-date-time.min.css',array());
        wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wc-order-limit-admin.css', array(), $this->version, 'all' );

    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    5.0.0
     */
    public function enqueue_scripts() {
        global $post;
        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Wc_Order_Limit_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Wc_Order_Limit_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_register_script('tether-js', 'https://npmcdn.com/tether@1.2.4/dist/js/tether.min.js');
        wp_register_script('bootstraps-js', plugin_dir_url( __FILE__ ).'js/bootstrap.min.js');
        if( (isset($_GET['page']) && $_GET['page'] == 'wcol_settings') || (isset($_GET['page']) && $_GET['page'] == 'wcol-support') ){
            wp_enqueue_script('tether-js');
            wp_enqueue_script('bootstraps-js');
        }
          
        wp_enqueue_script('select2-js',plugin_dir_url( __FILE__ ).'js/select2.full.min.js', array());
        wp_enqueue_script('jquery-rain-date-time-js', plugin_dir_url( __FILE__ ).'js/jquery-rain-date-time.js',array());
        wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/wc-order-limit-admin.js', array( 'jquery' ), $this->version, false );
        $script_vars = $this->get_wcol_script_vars();
        wp_localize_script( $this->plugin_name, 'wcol_script_vars', $script_vars );
        if(isset($post->post_type) && $post->post_type == 'wcol_rule'){
            wp_enqueue_script( $this->plugin_name.'-rule-type', plugin_dir_url( __FILE__ ) . 'js/wc-order-limit-rule.js', array( 'jquery' ), $this->version, true );
        }


    }

    public function wcol_add_admin_menu(){
        $license = get_option('xswcol_license',array());
        if(isset($license['status']) && $license['status']){
            add_submenu_page( 'edit.php?post_type=wcol_rule',esc_html__('Settings','wc-order-limit'), esc_html__('WC Order Settings','wc-order-limit'),'manage_options', 'wcol_settings', array($this, 'wcol_setting_callback'));
            add_submenu_page('edit.php?post_type=wcol_rule', esc_html__('Support' , 'wc-order-limit') , esc_html__('Support' , 'wc-order-limit'), 'manage_options', 'wcol-support', array($this , 'wcol_support_callback') );
        }else{
            add_menu_page(esc_html__('WC Order Limit','wc-order-limit'),esc_html__('WC Order Limit','wc-order-limit'),'manage_options','wcol-support',array($this, 'wcol_setting_callback_license'),'dashicons-lock',20);
        }

    }

    public function wcol_setting_callback_license(){
        include plugin_dir_path( __FILE__ ).'views/wc-order-limit-license.php';
    }
    public function wcol_support_callback(){
        include plugin_dir_path( __FILE__).'views/wc-order-limit-support.php';
    }
    public function wcol_create_post_type(){
        register_post_type( 'wcol_rule',
            array(
                'labels' => array(
                    'name'                  => esc_html__( 'Rules', 'wc-order-limit'),
                    'singular_name'         => esc_html__( 'Rule', 'wc-order-limit'),
                    'menu_name'             => _x( 'WC Order Limit', 'Admin menu name', 'wc-order-limit' ),
                    'all_items'             => __( 'Rules', 'wc-order-limit' ),
                    'add_new'               => __( 'Add Rule', 'wc-order-limit' ),
                    'add_new_item'          => __( 'Add New Rule', 'wc-order-limit' ),
                    'edit'                  => __( 'Edit', 'wc-order-limit' ),
                    'edit_item'             => __( 'Edit Rule', 'wc-order-limit' ),
                    'new_item'              => __( 'New Rule', 'wc-order-limit' ),
                    'search_items'          => __( 'Search Rules', 'wc-order-limit' ),
                    'not_found'             => __( 'No Rules found', 'woocommerce' ),
                    'not_found_in_trash'    => __( 'No Rules found in trash', 'wc-order-limit' ),
                ),
                'public'              => false,
                'show_ui'             => true,
                'map_meta_cap'        => true,
                'publicly_queryable'  => false,
                'exclude_from_search' => true,
                'show_in_menu'        => true,
                'hierarchical'        => false,
                'show_in_nav_menus'   => false,
                'rewrite'             => array('slug' => 'wcol_rule'),
                'query_var'           => false,
                'supports'            => array( 'title' ),
                'menu_position' => 20,
                'menu_icon'     => 'dashicons-lock',
     
            )
        );
        $defaults = array(
                    'wcol_settings'         => array(
                        'enable_store_limit'                        => "on",
                        'store_limit_message'                       => "{store-name} are closed now. It will open tommorrow 9am",
                        'store_message_total_amount'                => "{store-name} is sold max orders limit {max-order-limit} or max amount limit  {max-amount-limit} in {time-span}, Sale limit is reached. You will be able to place buy more after {limit-reset-day}. ",
                        'product_limit_message'                     =>  "{product-name} product minimum {applied-on} should be greater than {min-limit} and less than {max-limit}.",
                        'product_limit_message_across_all_orders'   =>  "You can buy maximum {max-limit} items of {product-name} {time-span}, your limit is reached. You will be able to place buy more after {limit-reset-day}.",
                        'product_limit_message_across_all_users_orders'     =>  "{product-name} is sold max limit {max-limit} .{product-name} is remaining items are {remaining}",
                        'product_limit_message_accomulative'        =>  "Following products accomulative {applied-on} should be greater than {min-limit} and less than {max-limit}.<br/>{product-names}.",
                        'category_limit_message'                    =>  "{category-name} category item minimum {applied-on} should be greater than {min-limit} and less than {max-limit}.",
                        'category_limit_message_across_all_orders'  =>  "You can buy maximum {max-limit} items of {category-name} {time-span}, your limit is reached. You will be able to place buy more after {limit-reset-day}.",
                        'category_limit_message_across_all_users_orders'    =>  "{category-name} is sold max limit are {max-limit} . {category-name} is remaining items are {remaining}.",
                        'category_limit_message_accomulative'       =>  "Following Categorys products accomulative {applied-on} should be greater than {min-limit} and less than {max-limit}.<br/>{category-names}.",
                        'vendor_limit_message'                      =>  "{vendor-shop-name} item minimum {applied-on} should be greater than {min-limit} and less than {max-limit}.",
                        'cart_total_limit_message'                  =>  "You must have an order with a minimum of {min-limit} and maximum of {max-limit} {applied-on} to place this order.",
                        'customer_message'                          =>  "You can place {rule-limit} order(s) {time-span}, your limit is reached. You will be able to place your order after {limit-reset-day}.",
                        'customer_message_total_amount'             =>  "You can order maximum of {rule-limit} amount {time-span}, your limit is reached. You will be able to place order after {limit-reset-day}.",
                        'enable_product_limit'                      =>  'on',
                        'enable_category_limit'                     =>  'on',
                        'enable_cart_total_limit'                   =>  'on',
                        'enable_checkout_button'                    =>  'on',
                        'enable_vendor_limit'                       =>  'on',
                        'enable_customer_limit'                     =>  'on',
                        'weekly_limit_reset_date'                   =>  'monday',
                        'monthly_limit_reset_date'                  =>  '1'
                    ),
                    'version'               => WC_ORDER_LIMIT_VERSION,
                    'wcol_exclude_rules'    => array()           
        );
        $wcol_options = get_option('wcol_options');
        if((isset($wcol_options['version']) && $wcol_options['version'] < WC_ORDER_LIMIT_VERSION) || (!isset($wcol_options['version']) && !empty($wcol_options))){
            if(isset($wcol_options['wcol_limit_rules']) && !empty($wcol_options['wcol_limit_rules'])){
                if(is_array($wcol_options['wcol_limit_rules'])){
                    foreach ($wcol_options['wcol_limit_rules'] as $key => $rule) {
                        if($rule['object_type'] == 'product'){
                            $object_type = 'products';
                        }
                        if($rule['object_type'] == 'product_cat'){
                            $object_type = 'categories';
                        }
                        if($rule['object_type'] == 'vendor'){
                            $object_type = 'vendor';
                        }
                        $post_arr = array(
                            'post_title'   => __($rule['object_type'].' rule '. $rule['wcol_min_order_limit'].' - '. $rule['wcol_max_order_limit']),
                            'post_status'   => 'publish',
                            'post_type'     => 'wcol_rule',
                            'meta_input'    => array(
                                'wcol-post-rules' => array(
                                    'rule-type'             => $object_type,
                                    'obj_ids'               => isset($rule['object_ids']) ? $rule['object_ids']:array(),
                                    'min-rule-limit'        => isset($rule['wcol_min_order_limit']) ? $rule['wcol_min_order_limit']:'',
                                    'applied_on'            => isset($rule['wcol_applied_on']) ? $rule['wcol_applied_on'] :'',
                                    'editable'              => isset($rule['editable']) ? $rule['editable'] :'',
                                    'disable-limit'         => isset($rule['disable-limit']) ? $rule['disable-limit'] :'',
                                    'accomulative'          => isset($rule['accomulative']) ? $rule['accomulative'] :'',
                                    'across-all-orders'     => isset($rule['across-all-orders']) ? $rule['across-all-orders'] :'',
                                    'enable-max-rule-limit' => isset($rule['enable-max-rule-limit']) ? $rule['enable-max-rule-limit'] :'',
                                    'max-rule-limit'        => isset($rule['wcol_max_order_limit']) ? $rule['wcol_max_order_limit'] :'',
                                    'enable-time-limit'     => isset($rule['enable_time_limit']) ? $rule['enable_time_limit'] :'',
                                    'rule-time-span'        => isset($rule['wcol_rule_time_span']) ? $rule['wcol_rule_time_span'] :'',
                                    'yearly-start-day'      => isset($rule['wcol_yearly_start_day']) ? $rule['wcol_yearly_start_day'] :'',
                                    'yearly-start-month'    => isset($rule['wcol_yearly_start_month']) ? $rule['wcol_yearly_start_month']:'',
                                    'weekly-start-day'      => isset($rule['wcol_weekly_start_day']) ? $rule['wcol_weekly_start_day']:'',
                                    'monthly-start-date'    => isset($rule['wcol_monthly_start_date']) ? $rule['wcol_monthly_start_date']:'',
                                    'rule-start-time'       => isset($rule['wcol_rule_start_time']) ? $rule['wcol_rule_start_time']:'',
                                    'rule-end-time'         => isset($rule['wcol_rule_end_time']) ? $rule['wcol_rule_end_time'] :'',
                                    'enable-for-users'      => isset($rule['enable_for_users']) ? $rule['enable_for_users'] :'',
                                    'user-type'             => isset($rule['wcol_rule_user_type']) ? $rule['wcol_rule_user_type']:'',
                                    'rule-roles'            => isset($rule['wcol_rule_roles']) ? $rule['wcol_rule_roles']:'',
                                    'rule-users'            => isset($rule['wcol_rule_users']) ? $rule['wcol_rule_users']:''
                                ),
                                'wcol-rule-type'            => $object_type
                            ),
                        );
                        $id = wp_insert_post($post_arr);
                    }
                }
            }
            if(isset($wcol_options['wcol_customer_rules']) && !empty($wcol_options['wcol_customer_rules'])){
                if(is_array($wcol_options['wcol_customer_rules'])){
                    foreach ($wcol_options['wcol_customer_rules'] as $key => $rule) {
                        $post_arr = array(
                            'post_title'   => __($rule['object_type'].' rule '. $rule['wcol_rule_limit']),
                            'post_status'   => 'publish',
                            'post_type'     => 'wcol_rule',
                            'meta_input'    => array(
                                'wcol-post-rules' => array(
                                    'rule-type'             => 'customer',
                                    'obj_ids'               => isset($rule['object_type']) ? array( $rule['object_type']): array(),
                                    'cus_applied_on'        => isset( $rule['wcol_applied_on']) ? $rule['wcol_applied_on'] :'',
                                    'disable-limit'         => isset( $rule['disable-limit']) ? $rule['disable-limit']:'',
                                    'cus-max-rule-limit'    => isset( $rule['wcol_rule_limit']) ? $rule['wcol_rule_limit']:'',
                                    'enable-time-limit'     => isset( $rule['enable_time_limit']) ? $rule['enable_time_limit']:'',
                                    'rule-time-span'        => isset( $rule['wcol_rule_time_span']) ? $rule['wcol_rule_time_span']:'',
                                    'yearly-start-day'      => isset( $rule['wcol_yearly_start_day']) ? $rule['wcol_yearly_start_day']:'',
                                    'yearly-start-month'    => isset( $rule['wcol_yearly_start_month']) ? $rule['wcol_yearly_start_month']:'',
                                    'weekly-start-day'      => isset( $rule['wcol_weekly_start_day']) ? $rule['wcol_weekly_start_day']:'',
                                    'monthly-start-date'    => isset( $rule['wcol_monthly_start_date']) ? $rule['wcol_monthly_start_date']:'',
                                    'rule-start-time'       => isset( $rule['wcol_rule_start_time']) ? $rule['wcol_rule_start_time']:'',
                                    'rule-end-time'         => isset( $rule['wcol_rule_end_time']) ? $rule['wcol_rule_end_time']:'',
                                    'cus-rule-users'        => isset( $rule['users']) ? $rule['users']:'',
                                    'cus-rule-roles'        => isset( $rule['roles']) ? $rule['roles']:'',
                                )
                            ),
                            'wcol-rule-type'                => $object_type
                        );
                        $id = wp_insert_post($post_arr);
                    }
                }
            }
            $new_wcol_options = array();
            $new_wcol_options['wcol_settings'] = $wcol_options['wcol_settings'];
            $new_wcol_options['version'] = WC_ORDER_LIMIT_VERSION;
            $new_wcol_options['wcol_exclude_rules'] = $wcol_options['wcol_exclude_rules'];
            update_option('wcol_options',$new_wcol_options);
        }
        if( empty($wcol_options) || !is_array($wcol_options) ){
            add_option('wcol_options', $defaults);
        }
    }

    public function wcol_register_metabox(){
        add_meta_box( 'wcol_rule_limits', esc_html__( 'Rule type', 'wc-order-limit' ), array($this,'wcol_rule_limits_callback') , 'wcol_rule' );
    }


    public function wcol_rule_limits_callback($post){
        $monthsArray = array(
            '01'    => esc_html__('Jan', 'wc-order-limit'),
            '02'    => esc_html__('Feb', 'wc-order-limit'),
            '03'    => esc_html__('Mar', 'wc-order-limit'),
            '04'    => esc_html__('Apr', 'wc-order-limit'),
            '05'    => esc_html__('May', 'wc-order-limit'),
            '06'    => esc_html__('Jun', 'wc-order-limit'),
            '07'    => esc_html__('Jul', 'wc-order-limit'),
            '08'    => esc_html__('Aug', 'wc-order-limit'),
            '09'    => esc_html__('Sep', 'wc-order-limit'),
            '10'    => esc_html__('Oct', 'wc-order-limit'),
            '11'    => esc_html__('Nov', 'wc-order-limit'),
            '12'    => esc_html__('Dec', 'wc-order-limit'),
        );
        $weeksArray = array(
            'monday'    => esc_html__('Monday', 'wc-order-limit'),
            'tuesday'   => esc_html__('Tuesday', 'wc-order-limit'),
            'wednesday' => esc_html__('Wednesday', 'wc-order-limit'),
            'thursday'  => esc_html__('Thursday', 'wc-order-limit'),
            'friday'    => esc_html__('Friday', 'wc-order-limit'),
            'saturday'  => esc_html__('Saturday', 'wc-order-limit'),
            'sunday'    => esc_html__('Sunday', 'wc-order-limit'),
        );
        $rule = get_post_meta($post->ID ,'wcol-post-rules',true);
        include plugin_dir_path( __FILE__ ).'views/wc-order-limit-rule-metabox-views.php';
    }

    public function wcol_save_metabox($post_id){
        if(isset($_POST['wcol-rules']) && get_post_type($post_id) == 'wcol_rule'){
            $rules = $this->wcol_sanitize($_POST['wcol-rules']);
            if($rules['rule-type'] == 'none'){
                update_option('wcol_rule_type_error',1);
                return false;
            }
            if(empty($rules['obj_ids'])){
                update_option('wcol_rule_select_items',1);
                return false;
            }
            if(empty($rules['min-rule-limit'])){
                $rules['min-rule-limit'] = 0;
            }
            update_post_meta($post_id,'wcol-post-rules',$rules);
            update_post_meta($post_id, 'wcol-rule-type',$rules['rule-type']);
        }

    }
    public function wcol_add_plugin_notice(){
        if (get_option('wcol_rule_type_error') == 1) { 
            // check whether to display the message
            add_action('admin_notices', array($this, 'rule_type_show_error'));
            // turn off the message
            update_option('wcol_rule_type_error', 0); 
        }
        if (get_option('wcol_rule_select_items') == 1) { 
            // check whether to display the message
            add_action('admin_notices', array($this, 'select_item_show_error'));
            // turn off the message
            update_option('wcol_rule_select_items', 0); 
        }
    }

    public function rule_type_show_error(){
        echo '<div class="error">
       <p>Please Select Rule Type</p>
       </div>';
    }

    public function select_item_show_error(){
        echo '<div class="error">
       <p>Please Select items</p>
       </div>';
    }
    public function wcol_sanitize($rules){
        foreach ( $rules as $key => $value) {
            if(is_array($value)){
                $rules[$key] = array_map('esc_html', $value);
            }else{
                $rules[$key] = esc_html($value);
            }
        }
        return $rules;
    }

    public function get_wcol_script_vars(){
        if(is_plugin_active( 'wc-vendors/class-wc-vendors.php' )){
            $vendor_plugin = 1;
        }else{
            $vendor_plugin = 0;
        }
        $script_var=array(
            'ajax_url'      =>  admin_url('admin-ajax.php'),
            'customer_type' =>  $this->get_customer_types(),
            'user_roles'    =>  $this->get_all_roles(),
            'memberships'   =>  $this->get_all_memberships(),
            'vendor_plugin' =>  $vendor_plugin
        );
        return  $script_var;
    }


    /*  get all wc products     */
    /*  @params null            */
    /*  returns array()         */
    public function wcol_get_product(){
        global $wpdb;
        $serach = isset($_GET['q'])?$_GET['q']:'';
        $limit = 10;
        $page = isset($_GET['page'])?$_GET['page']:1;
        $offset = $limit*($page-1);
        $query = "SELECT ID FROM {$wpdb->prefix}posts
        WHERE post_status IN ('pending', 'publish', 'private') AND post_type='product' AND post_name LIKE '%{$serach}%'";
        $total = $wpdb->get_results($query, ARRAY_A);
        $total_count = count($total);
        $product_query = "SELECT ID as id, post_title as text FROM {$wpdb->prefix}posts
                          WHERE post_status IN ('pending', 'publish', 'private') AND post_type='product'
                          AND post_name LIKE '%{$serach}%' ORDER BY post_title LIMIT {$limit} OFFSET {$offset}";
        $products = $wpdb->get_results($product_query, ARRAY_A);
        if(!isset($_GET['page'])){
            $products = array_merge(array(array('id' => '-1' , 'text' =>'All Products ')),$products);
        }
        $data = array('items'=> $products,'total_count' => $total_count);
        wp_send_json($data);
        wp_die();
    }
    
    /*  get all wc product categories   */
    /*  @params null                    */
    /*  returns array()                 */
    public function wcol_get_categories(){
        $serach = isset($_GET['q'])?$_GET['q']:'';
        $limit = 10;
        $page = isset($_GET['page'])?$_GET['page']:1;
        $offset = $limit*($page-1);
        $args = array(
            'taxonomy'     => 'product_cat',
            'fields'       => 'ids',
            'search'       => $serach,
            'orderby'      => 'name',
            'show_count'   => 0,
            'pad_counts'   => 0,
            'hierarchical' => 0,
            'hide_empty'   => 0
        );
        $count = get_terms( $args );
        $total_count = count($count);
        $args = array(
            'taxonomy'     => 'product_cat',
            'number'       => $limit,
            'offset'       => $offset,
            'fields'       => 'all',
            'search'       => $serach,
            'orderby'      => 'name',
            'show_count'   => 0,
            'pad_counts'   => 0,
            'hierarchical' => 0,
            'hide_empty'   => 0
        );
        $terms = get_terms( $args );
        $product_cat = array();
        if(!isset($_GET['page'])){
            $product_cat[] = array('id' => '-1' , 'text' =>'All Categories ');
        }
        if(is_array($terms)){
            foreach($terms as $term){
                $product_cat[]=array('id'=>$term->term_id,'text'=>$term->name);
            }
        }
        $data = array('items'=> $product_cat,'total_count' => $total_count);
        wp_send_json($data);
        wp_die();
    }

    
    
    /*  get all wc vendors  */
    /*  @params null        */
    /*  returns array()     */
    public function wcol_get_wc_vendors(){
        $serach = isset($_GET['q'])?$_GET['q']:'';
        $limit = 10;
        $page = isset($_GET['page'])?$_GET['page']:1;
        $offset = $limit*($page-1);
        $args = array(
            'role' => 'vendor',
            'orderby' => 'display_name',
            'search' => '*'.$serach.'*',
            'search_columns'=> array('display_name'),
            'order' => 'ASC',
            'fields' => array('ID' )
        );
        $total = get_users($args);
        $total_count = count($total);
        $args = array(
            'role' => 'vendor',
            'orderby' => 'user_nicename',
            'search' => '*'.$serach.'*',
            'search_columns'=> array('display_name'),
            'offset' => $offset,
            'number' => $limit,
            'order' => 'ASC',
            'fields' => array('ID','display_name' )
        );
        $users = get_users($args);
        $wc_vendors = array();
        if(!isset($_GET['page'])){
            $wc_vendors[] = array('id' => '-1' , 'text' =>'All Vendors');
        }
        if(is_array($users)){
            foreach($users as $user){
                $shop_name = get_user_meta($user->ID,'pv_shop_name',true);
                if($shop_name===''){
                    $shop_name = $user->display_name;
                }
                $wc_vendors[] = array('id'=>$user->ID, 'text'=>$shop_name);
            }
        }
        $data = array('items'=> $wc_vendors,'total_count' => $total_count);
        wp_send_json($data);
        wp_die();
    }
    
    /*  get all customer types  */
    /*  @params null        */
    /*  returns array()     */
    public function get_customer_types(){
        $customer_types = array(
            array('id' => 'all-users', 'text'=>__('All Users', 'wc-order-limit')),
            array('id' => 'every-user' , 'text' => __('Every Single User' , 'wc-order-limit')),
            array('id' => 'selective-users', 'text'=>__('Selective Users', 'wc-order-limit')),
            array('id' => 'roles', 'text'=>__('Roles','wc-order-limit')),
            array('id' => 'guest-users', 'text'=>__('Guest Users', 'wc-order-limit')),
        );
        return $customer_types;                 
    }
    
    /*  get all users   */
    /*  @params null        */
    /*  returns array()     */
    public function wcol_get_users(){
        global $wpdb;
        $serach = isset($_GET['q'])?$_GET['q']:'';
        $limit = 10;
        $page = isset($_GET['page'])?$_GET['page']:1;
        $offset = $limit*($page-1);
        $query = "SELECT ID FROM {$wpdb->prefix}users
        WHERE display_name LIKE '%{$serach}%'";
        $total = $wpdb->get_results($query, ARRAY_A);
        $total_count = count($total);
        $user_query = "SELECT ID as id, display_name as text FROM {$wpdb->prefix}users
                          WHERE display_name LIKE '%{$serach}%' ORDER BY display_name LIMIT {$limit} OFFSET {$offset}";
        $users = $wpdb->get_results($user_query, ARRAY_A);
        if(!isset($_GET['page'])){
            $users = array_merge(array(array('id' => '-1' , 'text' =>'All Users ')),$users);
        }
        $data = array('items'=> $users,'total_count' => $total_count);
        wp_send_json($data);
        wp_die();
    }
    
    /*  get all roles   */
    /*  @params null        */
    /*  returns array()     */
    public function get_all_roles(){
        $roles = array();
        global $wp_roles;
        if(is_array($wp_roles->roles)){
            foreach($wp_roles->roles as $key => $role){
                $roles[] = array('id'=>$key, 'text'=>$role['name']);
            }
        }
        return $roles;                  
    }

    public function get_all_memberships(){
        $memberships = array();
        if(is_plugin_active('woocommerce-memberships/woocommerce-memberships.php')){ 
            global $wpdb;
            $membership_query = "SELECT ID as id, post_title as text FROM {$wpdb->prefix}posts
                            WHERE post_status='publish' AND post_type='wc_membership_plan'
                            ORDER BY post_title";
            $memberships = $wpdb->get_results($membership_query, ARRAY_A);
        }
        return $memberships;
    }
    public function wcol_setting_callback(){
        $wcol_options = get_option('wcol_options');
        $wcol_settings = $wcol_options['wcol_settings'];
        $weeksArray = array(
            'monday'    => esc_html__('Monday', 'wc-order-limit'),
            'tuesday'   => esc_html__('Tuesday', 'wc-order-limit'),
            'wednesday' => esc_html__('Wednesday', 'wc-order-limit'),
            'thursday'  => esc_html__('Thursday', 'wc-order-limit'),
            'friday'    => esc_html__('Friday', 'wc-order-limit'),
            'saturday'  => esc_html__('Saturday', 'wc-order-limit'),
            'sunday'    => esc_html__('Sunday', 'wc-order-limit'),
        );
        $monthsArray = array(
            '01'    => __('Jan', 'wc-order-limit'),
            '02'    => __('Feb', 'wc-order-limit'),
            '03'    => __('Mar', 'wc-order-limit'),
            '04'    => __('Apr', 'wc-order-limit'),
            '05'    => __('May', 'wc-order-limit'),
            '06'    => __('Jun', 'wc-order-limit'),
            '07'    => __('Jul', 'wc-order-limit'),
            '08'    => __('Aug', 'wc-order-limit'),
            '09'    => __('Sep', 'wc-order-limit'),
            '10'    => __('Oct', 'wc-order-limit'),
            '11'    => __('Nov', 'wc-order-limit'),
            '12'    => __('Dec', 'wc-order-limit'),
        );
        ?>
        <div class="wrap wcol">
            <h3><?php esc_html_e('WC Order Limit Setting', 'wc-order-limit'); ?></h3>
            <div class="xs-wcol-alert xs-wcol-alert-success wcol-data-save-notice">
                <button type="button" class="xs-wcol-close xs-wcol-notice-dismiss" >&times;</button>
                <span></span>
            </div>
                <!-- Main Setting tab -->   
                <nav class="nav nav-tabs wcol-nav"> 
                    <!-- Order Limit Rules For Order Total  Tab --> 
                    <a id = "wcol-license-tab" href="?post_type=wcol_rule&page=wcol_settings&tab=wcol-store-tab" class="nav-tab <?php echo (!isset($_GET['tab']) || $_GET['tab'] == 'wcol-store-tab')? 'nav-tab-active' : '' ?>" ><?php _e('Store Limit','wc-order-limit'); ?></a>
                    <a id = "wcol-order-total" href="?post_type=wcol_rule&page=wcol_settings&tab=wcol-order-total-tab" class="nav-tab <?php echo (isset($_GET['tab']) &&  $_GET['tab'] == 'wcol-order-total-tab')? 'nav-tab-active' : '' ?>"  ><?php esc_html_e('Order Total','wc-order-limit'); ?></a>
                    <!-- Exlucde tab -->
                    <a id ="wcol-exclude-tab" href="?post_type=wcol_rule&page=wcol_settings&tab=wcol-exclude-tab" class="nav-tab <?php echo (isset($_GET['tab']) && $_GET['tab'] == 'wcol-exclude-tab')? 'nav-tab-active' : '' ?>" ><?php _e('Exclusion List','wc-order-limit'); ?></a>
                    <!-- Advance Tab -->                
                    <a id = "wcol-advance-tab" href="?post_type=wcol_rule&page=wcol_settings&tab=wcol-settings-tab" class="nav-tab <?php echo (isset($_GET['tab']) && $_GET['tab'] == 'wcol-settings-tab')? 'nav-tab-active' : '' ?>" ><?php esc_html_e('Advanced','wc-order-limt'); ?></a>
                    
                </nav>
                <div class="wcol-inner">
        <?php
            $wcol_exclude_rules = $wcol_options['wcol_exclude_rules'];
            $tab = 'wcol-store-tab';
            if(isset($_GET['tab'])){
                $tab = $_GET['tab'];
            }
            switch ($tab) {
                case 'wcol-exclude-tab':
                    include plugin_dir_path( __FILE__ ).'views/wc-order-limit-exclude.php';
                    break;
                case 'wcol-order-total-tab':
                    include plugin_dir_path( __FILE__ ).'views/wc-order-limit-settings.php';
                    break;
                case 'wcol-settings-tab':
                    include plugin_dir_path( __FILE__ ).'views/wc-order-limit-settings.php';
                    break;
                default:
                    include plugin_dir_path( __FILE__ ).'views/wc-order-limit-settings.php';
                    break;
            }
        
        ?>
                </div>
            <!-- </form> -->
            <!-- Modal -->
            <div class="modal fade xs-wcol-modal" id="wcol-modal" tabindex="-1" aria-labelledby="exampleModalLabel" aria-hidden="true">
                <div class="modal-dialog modal-dialog-centered">
                    <div class="modal-content">
                        <div class="modal-header">       
                          <h5 class="modal-title"><?php esc_html_e('Press Save button to save the Data','wc-order-limit'); ?></h5> 
                          <button type="button" class="close wcol-rule-button-disable" id='wcol-modal-close' data-dismiss="modal" aria-label="Close">
                            <span aria-hidden="true">&times;</span>
                          </button>
                        </div>
                        <div class="modal-body">
                            <h6><?php esc_html_e('Are you sure you want to leave without saving the Data?','wc-order-limit')?></h6>
                        </div>
                        <div class="modal-footer">
                            <button type="button" class="btn btn-secondary wcol-modal-disable" id='wcol-modal-pwos' data-dismiss="modal"><?php esc_html_e('No','wc-order-limit'); ?></button>
                            <button type="button" class="btn btn-success wcol-modal-disable" id='wcol-modal-sbp'><?php esc_html_e('Yes','wc-order-limit'); ?></button>
                        </div>
                    </div>
                </div>
            </div>
        </div>
        <?php
    }

    public function wc_order_limit_save_settings(){
        $data = array();
        $old_option = get_option('wcol_options',true);
        $exludes = isset($old_option['wcol_exclude_rules']) ? $old_option['wcol_exclude_rules']: array();
        $settings = isset($old_option['wcol_settings']) ? $old_option['wcol_settings']: array();
        parse_str($_POST['data'], $data);
        $raw_exclude_rules = (isset($data['wcol-rules']['exclude-rules']) && !empty($data['wcol-rules']['exclude-rules'])) ? $data['wcol-rules']['exclude-rules'] : array('object-ids'=>'') ;
        unset($data['_wp_http_referer']);
        $wcol_exclude_rules = array();
        if($raw_exclude_rules['object-ids']!='' && is_array($raw_exclude_rules['object-ids']) ){
            foreach($raw_exclude_rules['object-ids'] as $key=>$value){
                if($value!=''){
                    $wcol_exclude_rules[] = array(
                        'rule-id'                   =>  stripcslashes( $raw_exclude_rules['rule-id'][$key] ),
                        'disable-limit'             =>  stripcslashes( $raw_exclude_rules['disable-limit'][$key] ),
                        'object_ids'                =>  $value,
                        'object_type'               =>  'exclude_product',
                        'exclude-limit'             =>  stripcslashes( $raw_exclude_rules['enable'][$key] ),
                        'wcol_max_order_limit'      =>  stripcslashes( $raw_exclude_rules['rule-limit'][$key] )!='' ? stripslashes( $raw_exclude_rules['rule-limit'][$key] ) : 0,
                    );
                }
            }
        }        
        if(isset($data['settings-rules-changes'])){
            $data = $this->wcol_sanitize($data);
            $settings = $data;
        }
        
        if(isset($data['exculde_rules'])){
            $exludes = $wcol_exclude_rules;
        } 
        $new_option = array('wcol_settings' => $settings , 'wcol_exclude_rules' => $exludes);
        $res = update_option('wcol_options',$new_option);
        wp_send_json($res);

    }

    public function wcol_load_new_row(){
            ob_start();
            $monthsArray = array(
                '01'    => __('Jan', 'wc-order-limit'),
                '02'    => __('Feb', 'wc-order-limit'),
                '03'    => __('Mar', 'wc-order-limit'),
                '04'    => __('Apr', 'wc-order-limit'),
                '05'    => __('May', 'wc-order-limit'),
                '06'    => __('Jun', 'wc-order-limit'),
                '07'    => __('Jul', 'wc-order-limit'),
                '08'    => __('Aug', 'wc-order-limit'),
                '09'    => __('Sep', 'wc-order-limit'),
                '10'    => __('Oct', 'wc-order-limit'),
                '11'    => __('Nov', 'wc-order-limit'),
                '12'    => __('Dec', 'wc-order-limit'),
            );
            $weeksArray = array(
                'monday'    => __('Monday', 'wc-order-limit'),
                'tuesday'   => __('Tuesday', 'wc-order-limit'),
                'wednesday' => __('Wednesday', 'wc-order-limit'),
                'thursday'  => __('Thursday', 'wc-order-limit'),
                'friday'    => __('Friday', 'wc-order-limit'),
                'saturday'  => __('Saturday', 'wc-order-limit'),
                'sunday'    => __('Sunday', 'wc-order-limit'),
            );
            switch($_POST['rule_type']){
                case 'product' :
                    include plugin_dir_path( __FILE__ ).'views/template-single-product-rule-row.php';
                    break;
                case 'category' :
                    include plugin_dir_path( __FILE__ ).'views/template-single-product-cat-rule-row.php';
                    break;
                case 'vendor':
                    include plugin_dir_path( __FILE__ ).'views/template-single-vendor-rule-row.php';
                    break;
                case 'exclude':
                    include plugin_dir_path( __FILE__ ).'views/wc-order-limit-ajax-exclude.php';
                    break;
            }
            $new_row = ob_get_clean();
            echo $new_row;
            die();
        }
    /*  WCOL tab in product data panel  */
    /*  @params null                    */
    /*  returns null                    */
    public function WCOL_product_data_panel_tab(){
        ?><li class="wcol_tab"><a href="#wcol_tab"><?php esc_html_e('WC Order Limit','wc-order-limit'); ?></a></li><?php
    }

    /*  WCOL panel in product data panel    */
    /*  @params null                    */
    /*  returns null                    */
    public function WCOL_product_data_panel(){
        global $post;
        $plugin = new Wc_Order_Limit();
        $WCOL_Rule = new Wc_Order_limit_Rule ($plugin->get_plugin_name(),$plugin->get_version());
        if(isset($post->ID)){
            $wcol_rules = $WCOL_Rule->get_rules($post->ID, 'products');
        }else{
            $wcol_rules = array();
        }
        
        $monthsArray = array(
            '01'    => __('Jan', 'wc-order-limit'),
            '02'    => __('Feb', 'wc-order-limit'),
            '03'    => __('Mar', 'wc-order-limit'),
            '04'    => __('Apr', 'wc-order-limit'),
            '05'    => __('May', 'wc-order-limit'),
            '06'    => __('Jun', 'wc-order-limit'),
            '07'    => __('Jul', 'wc-order-limit'),
            '08'    => __('Aug', 'wc-order-limit'),
            '09'    => __('Sep', 'wc-order-limit'),
            '10'    => __('Oct', 'wc-order-limit'),
            '11'    => __('Nov', 'wc-order-limit'),
            '12'    => __('Dec', 'wc-order-limit'),
        );
        $weeksArray = array(
            'monday'    => __('Monday', 'wc-order-limit'),
            'tuesday'   => __('Tuesday', 'wc-order-limit'),
            'wednesday' => __('Wednesday', 'wc-order-limit'),
            'thursday'  => __('Thursday', 'wc-order-limit'),
            'friday'    => __('Friday', 'wc-order-limit'),
            'saturday'  => __('Saturday', 'wc-order-limit'),
            'sunday'    => __('Sunday', 'wc-order-limit'),
        );
        include plugin_dir_path( __FILE__ ).'views/wc-order-limit-rule-single-product-views.php';
    }

    public function WCOL_product_cat_fields($term){
        $plugin = new Wc_Order_Limit();
        $WCOL_Rule = new Wc_Order_limit_Rule ($plugin->get_plugin_name(),$plugin->get_version());
        if(isset($term->term_id)){
            $wcol_rules = $WCOL_Rule->get_rules($term->term_id, 'categories');
        }else{
            $wcol_rules = array();
        }
        
        $monthsArray = array(
            '01'    => __('Jan', 'wc-order-limit'),
            '02'    => __('Feb', 'wc-order-limit'),
            '03'    => __('Mar', 'wc-order-limit'),
            '04'    => __('Apr', 'wc-order-limit'),
            '05'    => __('May', 'wc-order-limit'),
            '06'    => __('Jun', 'wc-order-limit'),
            '07'    => __('Jul', 'wc-order-limit'),
            '08'    => __('Aug', 'wc-order-limit'),
            '09'    => __('Sep', 'wc-order-limit'),
            '10'    => __('Oct', 'wc-order-limit'),
            '11'    => __('Nov', 'wc-order-limit'),
            '12'    => __('Dec', 'wc-order-limit'),
        );
        $weeksArray = array(
            'monday'    => __('Monday', 'wc-order-limit'),
            'tuesday'   => __('Tuesday', 'wc-order-limit'),
            'wednesday' => __('Wednesday', 'wc-order-limit'),
            'thursday'  => __('Thursday', 'wc-order-limit'),
            'friday'    => __('Friday', 'wc-order-limit'),
            'saturday'  => __('Saturday', 'wc-order-limit'),
            'sunday'    => __('Sunday', 'wc-order-limit'),
        );
        include plugin_dir_path( __FILE__ ).'views/wc-order-limit-rule-single-cat-views.php';
    }

    public function process_product_meta_wcol_tab($post_id){
        $plugin = new Wc_Order_Limit();
        $WCOL_Rule = new Wc_Order_limit_Rule ($plugin->get_plugin_name(),$plugin->get_version());
        $WCOL_Rule->save_wcol_options($post_id, 'products');
    }

    public function save_wcol_product_cat_fields($term_id,$tt_id=0){
        $plugin = new Wc_Order_Limit();
        $WCOL_Rule = new Wc_Order_limit_Rule ($plugin->get_plugin_name(),$plugin->get_version());
        $WCOL_Rule->save_wcol_options($term_id, 'categories');
    }

    public function xswcol_delete_rule(){
        $rule_id = isset($_POST['rule_id']) ? $_POST['rule_id'] : '';
        if($rule_id){
            if(get_post_meta($rule_id,'wcol-post-rules',true)){
                wp_delete_post($rule_id);
            }
            wp_send_json(array('status' => true));
            wp_die();
        }else{
            wp_send_json(array('status'=>false));
            wp_die();
        }
    }
    public function WCOL_vendor_fields($user){
        $plugin = new Wc_Order_Limit();
        $WCOL_Rule = new Wc_Order_limit_Rule ($plugin->get_plugin_name(),$plugin->get_version());
        $user_meta=get_userdata($user->ID);
        $user_roles = $user_meta->roles;
        $wcol_rules = array();
        
        $monthsArray = array(
            '01'    => __('Jan', 'wc-order-limit'),
            '02'    => __('Feb', 'wc-order-limit'),
            '03'    => __('Mar', 'wc-order-limit'),
            '04'    => __('Apr', 'wc-order-limit'),
            '05'    => __('May', 'wc-order-limit'),
            '06'    => __('Jun', 'wc-order-limit'),
            '07'    => __('Jul', 'wc-order-limit'),
            '08'    => __('Aug', 'wc-order-limit'),
            '09'    => __('Sep', 'wc-order-limit'),
            '10'    => __('Oct', 'wc-order-limit'),
            '11'    => __('Nov', 'wc-order-limit'),
            '12'    => __('Dec', 'wc-order-limit'),
        );
        $weeksArray = array(
            'monday'    => __('Monday', 'wc-order-limit'),
            'tuesday'   => __('Tuesday', 'wc-order-limit'),
            'wednesday' => __('Wednesday', 'wc-order-limit'),
            'thursday'  => __('Thursday', 'wc-order-limit'),
            'friday'    => __('Friday', 'wc-order-limit'),
            'saturday'  => __('Saturday', 'wc-order-limit'),
            'sunday'    => __('Sunday', 'wc-order-limit'),
        );
        if(in_array('vendor', $user_roles)){
            if(isset($user->ID)){
                $wcol_rules = $WCOL_Rule->get_rules($user->ID, 'vendor');  
            }
            include plugin_dir_path( __FILE__ ).'views/wc-order-limit-rule-vendor-views.php';
        }
    }
    public function update_wcol_vendor_fields($vendor_id){
        $plugin = new Wc_Order_Limit();
        $WCOL_Rule = new Wc_Order_limit_Rule ($plugin->get_plugin_name(),$plugin->get_version());
        $WCOL_Rule->save_wcol_options($vendor_id, 'vendor');
    }
    public function xswcol_verify_purchase_code(){
        $data = array();
        parse_str($_POST['data'], $data);
        $code = $data['xswcol_pcode'];
        $email = $data['xswcol_email'];
        if (empty($code)) {
            $message['status'] = false;
            $message['message'] = "Please enter a purchase code.";
            wp_send_json($message);
            wp_die();
        }
        $personalToken = "6vuqbbbMqZFt7CAbz8wpoNxYRSuBh2Ky";
        $userAgent = "Purchase code verification on ".$_SERVER['SERVER_NAME'];

        // Surrounding whitespace can cause a 404 error, so trim it first
        $code = trim($code);

        $message = array();
        // Make sure the code looks valid before sending it to Envato
        if (!preg_match("/^([a-f0-9]{8})-(([a-f0-9]{4})-){3}([a-f0-9]{12})$/i", $code)) {
            $message['status'] = false;
            $message['message'] = "You have enter an invalid purchase code.";
            wp_send_json($message);
            wp_die();
        }
        $arg = array(
            'headers' => array(
                "Authorization" => "Bearer ".$personalToken,
                "User-Agent" => $userAgent,
            ),
        );
        $response = wp_remote_get("https://api.envato.com/v3/market/author/sale?code=".$code."", $arg);
        $api_response = json_decode( wp_remote_retrieve_body( $response ), true );
        if(isset($api_response['error'])){
             if ($api_response['error'] === 404) {
                $message['status'] = false;
                $message['message'] = "You have enter an invalid purchase code.";
                wp_send_json($message);
                wp_die();
            }

            // Anything other than HTTP 200 indicates a request or API error
            // In this case, you should again ask the user to try again later
            if ($api_response['error'] !== 200) {
                $message['status'] = false;
                $message['message'] = "Failed to validate code due to an error: HTTP {$api_response['error']}";
                wp_send_json($message);
                wp_die();
            }
        }

        if(isset($api_response['item'])){
            $params =  "license=".$api_response['license'];
            $params .= "&supported_until=".$api_response['supported_until'];
            $params .= "&buyer_name=".$api_response['buyer'];
            $params .= "&purchase_code=".$code;
            $params .= "&email=".$email;
            $params .= "&purchase_count=".$api_response['purchase_count'];
            $params .= "&client_host=".$_SERVER['SERVER_NAME'];
            $params .= "&ip=".$_SERVER['HTTP_CLIENT_IP'];
            $params .= "&id=".$api_response['item']['id'];
            $sever_response = wp_remote_get("https://www.xfinitysoft.com/wp-json/xslicense/v1/wcol/verify/?".$params);
            $se_api_response = json_decode( wp_remote_retrieve_body( $sever_response ), true );
            update_option("xswcol_pcode",$code);
            update_option("xswcol_email",$email);
            update_option("xswcol_license",$se_api_response);
            wp_send_json($se_api_response);
            wp_die();
        }
    }

    public function xswcol_delete_purchase_code(){
        $code = get_option("xswcol_pcode",false);
        if($code){
            $params  = "purchase_code=".$code;
            $params .= "&client_host=".$_SERVER['SERVER_NAME'];
            $params .= "&ip=".$_SERVER['HTTP_CLIENT_IP'];
            $sever_response = wp_remote_get("https://www.xfinitysoft.com/wp-json/xslicense/v1/wcol/delete/?".$params);

            $se_api_response = json_decode( wp_remote_retrieve_body( $sever_response ), true );
            if (isset($sever_response['error']) && $sever_response['error'] !== 200) {
                $message['status'] = false;
                $message['message'] = "Failed to validate code due to an error: HTTP {$api_response['error']}";
                wp_send_json($message);
                wp_die();
            }
            if(isset($se_api_response['status']) && $se_api_response['status']){
                delete_option("xswcol_license");
                wp_send_json($se_api_response);
                wp_die();
            }
        }
    }

    public function wcol_send_mail(){
        $data = array();
        parse_str($_POST['data'], $data);
        $data['plugin_name'] = 'WC Order Limit';
        $data['version'] = 'Pro Version';
        $data['website'] = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://".$_SERVER['HTTP_HOST'];
        $to = 'xfinitysoft@gmail.com';
        switch ($data['type']) {
            case 'report':
                $subject = 'Report a bug';
                break;
            case 'hire':
                $subject = 'Hire us';
                break;
            
            default:
                $subject = 'Request a Feature';
                break;
        }
        
        $body = '<!DOCTYPE html><html><body><table>';
        $body .='<tbody>';
        $body .='<tr><th>User Name</th><td>'.$data['wcol_name'].'</td></tr>';
        $body .='<tr><th>User email</th><td>'.$data['wcol_email'].'</td></tr>';
        $body .='<tr><th>Plugin Name</th><td>'.$data['plugin_name'].'</td></tr>';
        $body .='<tr><th>Version</th><td>'.$data['version'].'</td></tr>';
        $body .='<tr><th>Website</th><td><a href="'.$data['website'].'">'.$data['website'].'</a></td></tr>';
        $body .='<tr><th>Message</th><td>'.$data['wcol_message'].'</td></tr>';
        $body .='</tbody>';
        $body .='</table></body></html>';
        $admin_email = get_option( 'admin_email' );
        $headers = array( 'From: '.$data['wcol_name'].' <'.$admin_email.'>', 'Reply-To: '.$data['wcol_name'].' <'.$data['wcol_email'].'>','Content-Type: text/html; charset=UTF-8' );
        $params ="name=".$data['wcol_name'];
        $params.="&email=".$data['wcol_email'];
        $params.="&site=".$data['website'];
        $params.="&version=".$data['version'];
        $params.="&plugin_name=".$data['plugin_name'];
        $params.="&type=".$data['type'];
        $params.="&message=".$data['wcol_message'];
        $sever_response = wp_remote_post("https://xfinitysoft.com/wp-json/plugin/v1/quote/save/?".$params);
        $se_api_response = json_decode( wp_remote_retrieve_body( $sever_response ), true );
        
        if($se_api_response['status']){
            $mail = wp_mail( $to, $subject, $body, $headers );
            wp_send_json(array('status'=>true));
        }else{
            wp_send_json(array('status'=>false));
        }
        wp_die();
    }

    public function wcol_insert_posttitle($data, $postarr, $unsanitized_postarr){
        if($postarr['post_type'] == 'wcol_rule'){
            if(empty($data['post_title'])){
                $data['post_title'] = 'WCOL-'.$postarr['ID'];
            }
        }
        return $data;

    }

    public function wcol_add_filter($views){
        global $current_user, $wp_query,$wpdb;
        
        if(is_plugin_active( 'wc-vendors/class-wc-vendors.php' )){
            $query ="SELECT p.ID FROM $wpdb->posts p LEFT JOIN $wpdb->postmeta m ON p.ID = m.post_id WHERE p.post_type = 'wcol_rule' AND  m.meta_key = 'wcol-rule-type' AND  m.meta_value = 'vendor'";
            $result = $wpdb->get_results( $query, ARRAY_A );
            if(isset($_GET['rule_type']) && $_GET['rule_type'] == 'vendor'){
                $class = 'class="current"';
            }else{
                 $class = '';
            } 
            $views['vendor'] = sprintf(__('<a href="%s" '.$class.'>Vendors Rules <span class="count">(%d)</span></a>'), admin_url('edit.php?rule_type=vendor&post_type=wcol_rule'),count($result));
        }
        $query ="SELECT p.ID FROM $wpdb->posts p LEFT JOIN $wpdb->postmeta m ON p.ID = m.post_id WHERE p.post_type = 'wcol_rule' AND  m.meta_key = 'wcol-rule-type' AND  m.meta_value = 'products'";
        $result = $wpdb->get_results( $query, ARRAY_A );
        $class = (isset($_GET['rule_type']) && $_GET['rule_type'] == 'products') ? 'class="current"' : '';  
        $views['products'] = sprintf(__('<a href="%s" '.$class.'>Products Rules <span class="count">(%d)</span></a>'),admin_url('edit.php?rule_type=products&post_type=wcol_rule'),count($result));
        $query ="SELECT p.ID FROM $wpdb->posts p LEFT JOIN $wpdb->postmeta m ON p.ID = m.post_id WHERE p.post_type = 'wcol_rule' AND  m.meta_key = 'wcol-rule-type' AND  m.meta_value = 'categories'";
        $result = $wpdb->get_results( $query, ARRAY_A ); 
        $class = (isset($_GET['rule_type'] ) && $_GET['rule_type'] == 'categories' ) ? ' class="current"' : '';  
        $views['categories'] = sprintf(__('<a href="%s" ' .$class.'>Categories Rules <span class="count">(%d)</span></a>'),admin_url('edit.php?rule_type=categories&post_type=wcol_rule'), count($result) );
        $query = "SELECT p.ID FROM $wpdb->posts p LEFT JOIN $wpdb->postmeta m ON p.ID = m.post_id WHERE p.post_type = 'wcol_rule' AND  m.meta_key = 'wcol-rule-type' AND  m.meta_value = 'customer'";
        $result = $wpdb->get_results( $query, ARRAY_A );
        $class = (isset($_GET['rule_type']) && $_GET['rule_type'] == 'customer') ? ' class="current"' : '';  
        $views['customer'] = sprintf(__('<a href="%s" '.$class.'>Customers Rules <span class="count">(%d)</span></a>'),admin_url('edit.php?rule_type=customer&post_type=wcol_rule'),count($result));
        return $views;
    }

    public function wcol_filter_rule($query){
        global $pagenow;
        if(is_admin() && isset($_GET['post_type']) && $_GET['post_type'] == 'wcol_rule'){
            if($pagenow=='edit.php' && isset($_GET['rule_type']) && $_GET['rule_type'] != ''){
                $query->query_vars['meta_key'] = 'wcol-rule-type';
                $query->query_vars['meta_value'] = $_GET['rule_type'];
            }

        }

    }

    public function wcol_updated_messages($messages){
        $post             = get_post();
        $post_type        = get_post_type( $post );
        $post_type_object = get_post_type_object( $post_type );
     
        $messages['wcol_rule'] = array(
            0  => '', // Unused. Messages start at index 1.
            1  => __( 'Rule updated.', 'wc-order-limit' ),
            2  => __( 'Custom field updated.', 'wc-order-limit' ),
            3  => __( 'Custom field deleted.', 'wc-order-limit' ),
            4  => __( 'Rule updated.', 'textdomain' ),
            /* translators: %s: date and time of the revision */
            5  => isset( $_GET['revision'] ) ? sprintf( __( 'Rule restored to revision from %s', 'wc-order-limit' ), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false,
            6  => __( 'Rule published.', 'wc-order-limit' ),
            7  => __( 'Rule saved.', 'wc-order-limit' ),
            8  => __( 'Rule submitted.', 'wc-order-limit' ),
            9  => sprintf(
                __( 'Rule scheduled for: <strong>%1$s</strong>.', 'wc-order-limit' ),
                // translators: Publish box date format, see http://php.net/date
                date_i18n( __( 'M j, Y @ G:i', 'wc-order-limit' ), strtotime( $post->post_date ) )
            ),
            10 => __( 'Rule draft updated.', 'wc-order-limit' ),
        );
        return $messages;
    }

    public function xswcol_variation_options($loop, $variation_data, $variation){
        $is_checked = get_post_meta( $variation->ID, '_wcol_rule_limit', true );
        if ( $is_checked == 'yes' ) {
            $is_checked = 'checked';
        } else {
            $is_checked = '';     
        }
        ?>
        <label class="tips" data-tip="<?php esc_attr_e( 'Apply Min and max limit in variation', 'woocommerce' ); ?>">
            <?php esc_html_e( 'Apply Min/Max limit', 'woocommerce' ); ?>
            <input type="checkbox" data-id='<?php echo esc_attr( $loop ); ?>' class="checkbox variable_checkbox wcol_rule_checkbox" name="_wcol_rule_limit[<?php echo esc_attr( $loop ); ?>]"<?php echo $is_checked; ?>/>
        </label>
        <?php
    }
    public function xswcol_add_limit_field_to_variations($loop, $variation_data, $variation){
        $is_checked = get_post_meta( $variation->ID, '_wcol_rule_limit', true );
        if ( $is_checked == 'yes' ) {
            $class = '';
        } else {
            $class = 'wcol-hidden';     
        }
        woocommerce_wp_text_input( array(
                'id' => 'wcol_min_limit_' . $loop,
                'class' => 'short ',
                'wrapper_class' =>$class,
                'label' => __( 'Minimum Limit', 'woocommerce' ),
                'value' => get_post_meta( $variation->ID, 'wcol_min_limit', true )
            ) 
        );
        woocommerce_wp_text_input( array(
            'id' => 'wcol_max_limit_' . $loop,
            'class' => 'short ',
            'wrapper_class' => $class,
            'label' => __( 'Maximum Limit', 'woocommerce' ),
            'value' => get_post_meta( $variation->ID, 'wcol_max_limit', true )
            ) 
        );
    }
    public function xswcol_add_limit_field_variation_data($variations ){
        $variations['wcol_min_limit'] = '<div class="woocommerce_wcol_min_limit">Minimum limit: <span>' . get_post_meta( $variations[ 'variation_id' ], 'wcol_min_limit', true ) . '</span></div>';
        $variations['wcol_max_limit'] = '<div class="woocommerce_wcol_min_limit">Maximum limit: <span>' . get_post_meta( $variations[ 'variation_id' ], 'wcol_min_limit', true ) . '</span></div>';
        return $variations;
    }
    public function xswcol_save_product_variation($variation_id, $i){
        if ( ! empty( $_POST['_wcol_rule_limit'] ) && ! empty( $_POST['_wcol_rule_limit'][$i] ) ) {
            update_post_meta( $variation_id, '_wcol_rule_limit', 'yes' );
        } else {
            update_post_meta( $variation_id, '_wcol_rule_limit', 'no' ); 
        }
        $wcol_min_limit = $_POST['wcol_min_limit_'.$i];
        if ( isset( $wcol_min_limit) ) update_post_meta( $variation_id, 'wcol_min_limit', esc_attr( $wcol_min_limit ) ); 
        $wcol_max_limit = $_POST['wcol_max_limit_'.$i];
        if ( isset( $wcol_max_limit) ) update_post_meta( $variation_id, 'wcol_max_limit', esc_attr( $wcol_max_limit ) );  
    }
}
