<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;   //Exit if accessed directly.
}
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://xfinitysoft.com/
 * @since      5.0.0
 *
 * @package    Wc_Order_Limit
 * @subpackage Wc_Order_Limit/includes
 */
 
/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wc_Order_Limit
 * @subpackage Wc_Order_Limit/includes
 * @author     xfinitysoft <support@xfinitysoft.com>
 */
class Wc_Order_limit_Rule {

	/**
	 * The ID of this plugin.
	 *
	 * @since    5.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    5.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    5.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/*	GET All Rules	*/
	/*	@params	null	*/
	/*	returns	Array	*/
	public function get_rules($object_id, $object_type){
		$args = array(
			'fields' => 'ids',
		    'post_type'=> 'wcol_rule',
		    'status'    => 'publish',
		    'numberposts' => -1,
		);              
		$rule_ids = get_posts($args);
		$match_rules = array();
		foreach ($rule_ids as $id) {
			$rules = get_post_meta($id,'wcol-post-rules',true);
			$rules['id'] = $id;
			
			if($object_type == $rules['rule-type']){
				
				if($object_id == "accomulative" && isset($rules['accomulative']) && $rules['accomulative']  == 'on'){
					$match_rules[] = $rules;
				}else{
					if( in_array($object_id, $rules['obj_ids']) || in_array( "-1" , $rules['obj_ids']) ){
						
						$match_rules[] = $rules;
					}
				}
				

			} 

		}
		
		if(empty($match_rules)){
			return false;
		}else{
			return $match_rules;
		}
	}
	/*	Group cart totals by product cat	*/
	/*	@params	null						*/
	/*	returns	Array						*/
	public function cart_totals_by_product_cat(){
		$product_cats = array();
		foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ) {
			$_product     = apply_filters( 'woocommerce_cart_item_product', $cart_item['data'], $cart_item, $cart_item_key );
			//$product_id = $_product->post->ID;
			$product_id = $cart_item['product_id'];
			$terms = wp_get_post_terms( $product_id, 'product_cat');
			if(is_array($terms)){
				foreach($terms as $term){
					$product_cats[$term->term_id][]=$cart_item;
				}
			}
		}
		$product_cats_total=array();
		if(is_array($product_cats)){
			foreach($product_cats as $term_id => $items){
				$amount = 0;
				$qty=0;
				if(is_array($items)){
					foreach($items as $item){
						$amount+=apply_filters('wcol_cart_item_total', $item['line_total'], $item, 'product_cat');
						$qty += apply_filters('wcol_cart_item_qty', $item['quantity'], $item, 'product_cat');
					}
				}
				$arg = array('amount'=>$amount, 'quantity'=>$qty);
				$product_cats_total[$term_id] = apply_filters('xswcol_product_total' , $arg , $items );
			}
		}
		return $product_cats_total;
	}

	/*	Group cart totals by vendor	*/
	/*	@params	null				*/
	/*	returns	Array				*/
	public function cart_totals_by_vendor(){
		$vendors = array();

		foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ) {
			$_product   = apply_filters( 'woocommerce_cart_item_product', $cart_item['data'], $cart_item, $cart_item_key );
			$product_id = $_product->get_id();
			$post = get_post($product_id);
			if( $post->post_parent > 0 ){
				$post = get_post( $post->post_parent );
			}
			
			$vendors[$post->post_author][] = $cart_item;
		}
		$vendors_total=array();
		if(is_array($vendors)){
			foreach($vendors as $vendor_id => $items){
				$amount = 0;
				$qty=0;
				if(is_array($items)){
					foreach($items as $item){
						$amount+=apply_filters('wcol_cart_item_total', $item['line_total'], $item, 'vendor');
						$qty += apply_filters('wcol_cart_item_qty', $item['quantity'], $item, 'vendor');
					}
				}
				$arg = array('amount'=>$amount, 'quantity'=>$qty);
				$vendors_total[$vendor_id] = apply_filters('xswcol_product_total' , $arg , $items );
			}
		}
		return $vendors_total;
	}

	public function wc_order_limit_check(){
		$wcol_settings = $this->get_wcol_settings();
		WC()->session->set('is_valid_order', true);
		$default_time_zone = date_default_timezone_get();
		$product_cat_total = $this->cart_totals_by_product_cat();
		$vendor_totals = $this->cart_totals_by_vendor();
		$current_coupons = WC()->cart->get_applied_coupons();
		$coupon_amounts=0;
		$date_format = apply_filters('xswcol_date_format','d M Y h:i:s a');
		if(!empty($current_coupons)){
			foreach ($current_coupons as $key => $value) {
				$coupon = new WC_Coupon($value);
				$coupon_amounts = $coupon_amounts + $coupon->get_amount();
			}
		}
		/*	*	*	*	*	*	*	*	*	*	*/
		/* 	Check for Store total Order Limit	*/
		/*	*	*	*	*	*	*	*	*	*	*/
		if(isset($wcol_settings['enable_store_limit'])  && $wcol_settings['enable_store_limit']=='on' ){
			if(isset($wcol_settings['cart_store_limit']) && $wcol_settings['cart_store_limit'] == 'on'){
				$message = $wcol_settings['store_limit_message'];
				$message = str_replace(
					array(
						'{store-name}'
					),
					array(
						get_bloginfo( 'name' )
					),
					$message
				);
				if( is_cart() ) {
					wc_print_notice(apply_filters('store_limit_notice', $message,  $wcol_settings), 'error');
				} else {
					wc_add_notice(apply_filters('store_limit_notice', $message, $wcol_settings), 'error');
				}
				WC()->session->set('is_valid_order', false);
			}
			$applied = false;
			$applied = $this->valid_store_limit($wcol_settings);
			if($applied){
				$max_value = $wcol_settings['store_max_limit'];
				$max_amount = wc_price($wcol_settings['store_max_amount_limit']);
				$message = $wcol_settings['store_message_total_amount'];
				switch($wcol_settings['wcol_store_time_span']){
					case 'daily':
						$time_span_text = __('Daily', 'wc-order-limit');
						break;
					case 'weekly':
						$time_span_text = __('Weekly', 'wc-order-limit');
						break;
					case 'monthly':
						$time_span_text = __('Monthly', 'wc-order-limit');
						break;
					case 'yearly':
						$time_span_text = __('Yearly', 'wc-order-limit');
						break;	
					case 'custom':
						$time_span_text = __('from ', 'wc-order-limit').date('d M Y h:i a', $applied['from']). __(' to ', wc-order-limit).date('d M Y h:i a', $applied['to']);
						break;
				}
				$message = str_replace(
					array(
						'{store-name}',
						'{max-order-limit}',
						'{max-amount-limit}',
						'{time-span}',
						'{limit-reset-day}',

					),
					array(
						get_bloginfo( 'name' ),
						$max_value,
						$max_amount,
						$time_span_text,
						date($date_format, $applied['to'])
					),
					$message
				);
				if( is_cart() ) {
					wc_print_notice(apply_filters('store_limit_notice', $message,  $wcol_settings), 'error');
				} else {
					wc_add_notice(apply_filters('store_limit_notice', $message, $wcol_settings), 'error');
				}
				WC()->session->set('is_valid_order', false);
			}

		}

		/*	*	*	*	*	*	*	*	*	*	*/
		/* 	Check for Cart total Order Limit	*/
		/*	*	*	*	*	*	*	*	*	*	*/
		if ( isset($wcol_settings['enable_cart_total_limit']) && $wcol_settings['enable_cart_total_limit']=='on') {
			$applied = false;			
			$applied_on = isset($wcol_settings['cart_total_applied_on'])?$wcol_settings['cart_total_applied_on']:'';
			switch($applied_on){
				case 'amount':
						$cart_total_amount = apply_filters( 'wcol_cart_total_amount', WC()->cart->total-WC()->cart->fee_total, $wcol_settings, WC()->cart );
						if(isset($wcol_settings['cart_total_enable_maximum_limit']) && $wcol_settings['cart_total_enable_maximum_limit']=='on' && (($cart_total_amount+$coupon_amounts) < $wcol_settings['cart_total_minimum_limit'] || $cart_total_amount > $wcol_settings['cart_total_maximum_limit']) ){
							$applied = true;
							$min_value = wc_price($wcol_settings['cart_total_minimum_limit']);
							$max_value = wc_price($wcol_settings['cart_total_maximum_limit']);
						}elseif(($cart_total_amount+$coupon_amounts) < $wcol_settings['cart_total_minimum_limit']){
							$applied = true;
							$min_value = wc_price($wcol_settings['cart_total_minimum_limit']);
							$max_value = wc_price($wcol_settings['cart_total_maximum_limit']);
						}else{
							$applied = false;
						}
						break;
				case 'quantity':
						$cart_contents_count = apply_filters( 'wcol_cart_contents_count', WC()->cart->get_cart_contents_count(), $wcol_settings, WC()->cart );
						if(isset($wcol_settings['cart_total_enable_maximum_limit']) && $wcol_settings['cart_total_enable_maximum_limit']=='on' && ($cart_contents_count < $wcol_settings['cart_total_minimum_limit'] || $cart_contents_count > $wcol_settings['cart_total_maximum_limit']) ){
							$applied = true;
							$min_value = $wcol_settings['cart_total_minimum_limit'];
							$max_value = $wcol_settings['cart_total_maximum_limit'] != '' ? $wcol_settings['cart_total_maximum_limit'] : '';
						}elseif($cart_contents_count < $wcol_settings['cart_total_minimum_limit']){
							$applied = true;
							$min_value = $wcol_settings['cart_total_minimum_limit'];
							$max_value = $wcol_settings['cart_total_maximum_limit'];
						}else{
							$applied = false;
						}
						break;
			}
			if( isset($wcol_settings['cart_total_enable_single_cat_limit']) && $wcol_settings['cart_total_enable_single_cat_limit']=='on'){
				$cat_ids = array();
				foreach ( wc()->cart->get_cart() as $cart_item_key => $cart_item ) {
				    $cat_ids = array_merge( $cat_ids, $cart_item['data']->get_category_ids() );
				}
				$count = count($cat_ids);
				if($count > 1){
					$applied = true;
				}
				
			}
			if($applied){
				if($wcol_settings['cart_total_enable_maximum_limit']!='on'){
					$max_value = '<span style="font-size:30px; font-weight:bold;vertical-align: middle;">∞</span>';
				}
				if(isset($wcol_settings['cart_total_enable_single_cat_limit']) && $wcol_settings['cart_total_enable_single_cat_limit']=='on'){
					$message = $wcol_settings['cart_single_cat_limit_message'];
				}else{
					$message = $wcol_settings['cart_total_limit_message'];
					$message = str_replace(
								array(
									 '{min-limit}',
									 '{max-limit}',
									 '{applied-on}'
								),
								array(
									$min_value,
									$max_value ,
									$applied_on
								),
								$message
							);
				}
				if( is_cart() ) {
					wc_print_notice(apply_filters('order_total_order_limit_notice', $message,  $wcol_settings), 'error');
				} else {
					wc_add_notice(apply_filters('order_total_order_limit_notice', $message, $wcol_settings), 'error');
				}
				WC()->session->set('is_valid_order', false);
			}
		}

		/*	*	*	*	*	*	*	*	*	*	*	*/
		/* 	Check for Vendor Base Rules Order Limit	*/
		/*	*	*	*	*	*	*	*	*	*	*	*/
		if(isset($wcol_settings['enable_vendor_limit']) && $wcol_settings['enable_vendor_limit']=='on' && is_array($vendor_totals) ){
			foreach($vendor_totals as $vendor_id=>$value){
				$rules_applied = $this->is_valid_order($vendor_id, 'vendor', $value,$coupon_amounts);
				if(is_array($rules_applied)){
					foreach($rules_applied as $rule){
						if($rule['applied_on']=='amount'){
							$wcol_min_value = wc_price( $rule['min-rule-limit'] );
							$wcol_max_value = $rule['max-rule-limit'] != '' ? wc_price( $rule['max-rule-limit'] ) : '';
						}else{
							$wcol_min_value = $rule['min-rule-limit'];
							$wcol_max_value = $rule['max-rule-limit'] != '' ? $rule['max-rule-limit'] : '';
						}
						$vendor_shop_name = get_user_meta($vendor_id, 'pv_shop_name', true);
						
						if($rule['enable-max-rule-limit']!='on'){
							$wcol_max_value = '<span style="font-size:30px; font-weight:bold;vertical-align: middle;">∞</span>';
						}
							
						$message = $wcol_settings['vendor_limit_message'];
						switch($rule['rule-time-span']){
							case 'daily':
								$time_span_text = __('Daily', 'wc-order-limit');
								break;
							case 'weekly':
								$time_span_text = __('Weekly', 'wc-order-limit');
								break;
							case 'monthly':
								$time_span_text = __('Monthly', 'wc-order-limit');
								break;
							case 'yearly':
								$time_span_text = __('Yearly', 'wc-order-limit');
								break;	
							case 'custom':
								$time_span_text = __('from ', 'wc-order-limit').date('d M Y h:i a', $rule['from']). __(' to ', 'wc-order-limit').date('d M Y h:i a', $rule['to']);
								break;
						}
						$message = str_replace(
								array(
									'{vendor-shop-name}',
									'{min-limit}',
									'{max-limit}',
									'{applied-on}',
									'{time-span}',
									'{limit-reset-day}',
									'{remaining}',
									'{endline}',
								),
								array(
									$vendor_shop_name,
									$wcol_min_value,
									$wcol_max_value,
									$rule['applied_on'],
									$time_span_text,
									date($date_format, $rule['to']),
									$rule['remaining'],
									'<br>',
								),
								$message
							);
						if( is_cart() ) {
							wc_print_notice( apply_filters('vendor_order_limit_notice', $message, $vendor_id, $rule), 'error');
						} else {
							wc_add_notice( apply_filters('vendor_order_limit_notice', $message, $vendor_id, $rule), 'error' );
						}
						WC()->session->set('is_valid_order', false);
					}
				}
			}
		}

		/*	*	*	*	*	*	*	*	*	*	*	*	*/
		/* 	Check for Product Base Rules Order Limit	*/
		/*	*	*	*	*	*	*	*	*	*	*	*	*/
		if(isset($wcol_settings['enable_product_limit']) && $wcol_settings['enable_product_limit']=='on'){
			foreach(WC()->cart->get_cart() as $cart_item_key=>$cart){
				$_product   = apply_filters( 'woocommerce_cart_item_product', $cart['data'], $cart, $cart_item_key );
				$product_id = $_product->get_id();
				
				$xs_pro_value = array();
				$xs_pro_value['quantity'] = apply_filters('wcol_cart_item_qty', $cart['quantity'], $cart, 'product');
				$xs_pro_value['amount'] = apply_filters('wcol_cart_item_total', $cart['line_total'], $cart, 'product');
				if($this->check_exclude_rule($product_id)){
					continue;
				}
				$cart_vid = null;
				if(isset($cart['variation_id']) && !empty($cart['variation_id'])){
					$cart_vid = $cart['variation_id'];
					$product_id = wp_get_post_parent_id($cart_vid);
					$is_checked = get_post_meta($cart_vid, '_wcol_rule_limit', true );
					if($is_checked == 'yes'){
						$_product = wc_get_product($cart_vid);
					}
				}
				
				$rules_applied = $this->is_valid_order($product_id, 'products', $xs_pro_value,$coupon_amounts,$cart_vid);
				if(is_array($rules_applied)){
					foreach($rules_applied as $rule){
						if($rule['applied_on']=='amount'){
							$wcol_min_value = wc_price( $rule['min-rule-limit'] );
							$wcol_max_value = $rule['max-rule-limit'] != '' ? wc_price( $rule['max-rule-limit'] ) : '';
						}else{
							$wcol_min_value = $rule['min-rule-limit'];
							$wcol_max_value = $rule['max-rule-limit'] != '' ? $rule['max-rule-limit'] : '';
						}
						if($rule['enable-max-rule-limit']!='on'){
							$wcol_max_value = '<span style="font-size:30px; font-weight:bold;vertical-align: middle;">∞</span>';
						}
						if($rule['across-all-orders']  !='1' and $rule['applied_for'] != 'minimum_limit'){
							if($rule['across-all-orders'] == '2'){
		                        $message = $wcol_settings['product_limit_message_across_all_orders'];
                            }else{
                            	$message = $wcol_settings['product_limit_message_across_all_users_orders'];
                            }
							switch($rule['rule-time-span']){
								case 'daily':
									$time_span_text = __('Daily', 'wc-order-limit');
									break;
								case 'weekly':
									$time_span_text = __('Weekly', 'wc-order-limit');
									break;
								case 'monthly':
									$time_span_text = __('Monthly', 'wc-order-limit');
									break;
								case 'yearly':
									$time_span_text = __('Yearly', 'wc-order-limit');
									break;	
								case 'custom':
									$time_span_text = __('from ', 'wc-order-limit').date('d M Y h:i a', $rule['from']). __(' to ', 'wc-order-limit').date('d M Y h:i a', $rule['to']);
									break;
							}
							$message = str_replace(
									array(
										'{product-name}',
										'{min-limit}',
										'{max-limit}',
										'{applied-on}',
										'{time-span}',
										'{limit-reset-day}',
										'{remaining}',
										'{endline}',
									),
									array(
										$_product->get_name(),
										$wcol_min_value,
										$wcol_max_value,
										$rule['applied_on'],
										$time_span_text,
										date($date_format, $rule['to']),
										$rule['remaining'],
										'<br>',
									),
									$message
								);
						}else{
							switch($rule['rule-time-span']){
								case 'daily':
									$time_span_text = esc_html__('Daily', 'wc-order-limit');
									break;
								case 'weekly':
									$time_span_text = esc_html__('Weekly', 'wc-order-limit');
									break;
								case 'monthly':
									$time_span_text = esc_html__('Monthly', 'wc-order-limit');
									break;
								case 'yearly':
									$time_span_text = esc_html__('Yearly', 'wc-order-limit');
									break;	
								case 'custom':
									$time_span_text = esc_html__('from ', 'wc-order-limit').date('d M Y h:i a', strtotime($rule['wcol_rule_start_time'])). esc_html__(' to ', 'wc-order-limit').date('d M Y h:i a', strtotime($rule['wcol_rule_end_time']));
									break;
							}
							$message = $wcol_settings['product_limit_message'];
							$time_span =  $this->wcol_get_time_span($rule, $this->get_wcol_settings());
							$message = str_replace(
									array(
										'{product-name}',
										'{min-limit}',
										'{max-limit}',
										'{time-span}',
										'{applied-on}',
										'{limit-reset-day}',
										'{remaining}',
										'{endline}',
									),
									array(
										$_product->get_title(),
										$wcol_min_value,
										$wcol_max_value,
										$time_span_text,
										$rule['applied_on'],
										date('d M Y h:i:s a', $time_span['to']),
										$rule['remaining'],
										'<br>',
									),
									$message
								);
						}
						if( is_cart() ) {
							wc_print_notice(apply_filters('product_order_limit_notice', $message, $product_id, $rule), 'error');
						} else {
							wc_add_notice(apply_filters('product_order_limit_notice', $message, $product_id, $rule), 'error');
						}
						WC()->session->set('is_valid_order', false);
						
					}
				}
			}
		}

		/*	*	*	*	*	*	*	*	*	*	*/
		/*	 Accomulative Checks for Products	*/
		/*	*	*	*	*	*	*	*	*	*	*/
		if(isset($wcol_settings['enable_product_limit']) && $wcol_settings['enable_product_limit']=='on' && is_array($this->get_accomulative_rules_on_cart('products',$coupon_amounts) ) ){
			foreach($this->get_accomulative_rules_on_cart('products',$coupon_amounts) as $rule){
				if(isset( $rule['applied_on'] ) && $rule[$rule['applied_on']] > 0  ){
					$products = array();
					if( is_array($rule['obj_ids']) ){
						foreach($rule['obj_ids'] as $object_id){
								if($object_id != "-1"){
									$products[] = get_the_title($object_id);
								}else{
									$products[] = "All Products";
								}	
						}
					}
					if($rule['applied_on']=='amount'){
						$wcol_min_value = wc_price( $rule['min-rule-limit'] );
						$wcol_max_value = $rule['max-rule-limit'] != '' ? wc_price( $rule['max-rule-limit'] ) : '';
					}else{
						$wcol_min_value = $rule['min-rule-limit'];
						$wcol_max_value = $rule['max-rule-limit'];
					}
					if(empty($wcol_max_value) || $rule['enable-max-rule-limit']!='on'){
						$wcol_max_value = '<span style="font-size:30px; font-weight:bold;vertical-align: middle;">∞</span>';
					}
					
					$message = $wcol_settings['product_limit_message_accomulative'];
					switch($rule['rule-time-span']){
						case 'daily':
							$time_span_text = esc_html__('Daily', 'wc-order-limit');
							break;
						case 'weekly':
							$time_span_text = esc_html__('Weekly','wc-order-limit');
							break;
						case 'monthly':
							$time_span_text = esc_html__('Monthly', 'wc-order-limit');
							break;
						case 'yearly':
							$time_span_text = esc_html__('Yearly', 'wc-order-limit');
							break;	
						case 'custom':
							$time_span_text = esc_html__('from ','wc-order-limit').date('d M Y h:i a', $rule['from']). esc_html__(' to ', 'wc-order-limit').date('d M Y h:i a', $rule['to']);
							break;
					}
					$message = str_replace(
							array(
								'{product-names}',
								'{min-limit}',
								'{max-limit}',
								'{applied-on}',
								'{time-span}',
								'{limit-reset-day}',
								'{remaining}',
								'{endline}'
							),
							array(
								implode(',', $products),
								$wcol_min_value,
								$wcol_max_value,
								$rule['applied_on'],
								$time_span_text,
								date($date_format, $rule['to']),
								$rule['remaining'],
								'<br>'
							),
							$message
						);

					if( is_cart() ) {
						wc_print_notice( apply_filters('accomulative_products_limit_notice', $message, $rule), 'error' );
					} else {
						wc_add_notice( apply_filters('accomulative_products_limit_notice', $message, $rule), 'error' );
					}
					WC()->session->set('is_valid_order', false);
				}
			}
		}

		/*	*	*	*	*	*	*	*	*	*	*	*	*/
		/* 	Check for Category Base Rules Order Limit	*/
		/*	*	*	*	*	*	*	*	*	*	*	*	*/
		if(isset($wcol_settings['enable_category_limit']) && $wcol_settings['enable_category_limit']=='on' && is_array($product_cat_total) ){
			foreach($product_cat_total as $term_id=>$value){
				$rules_applied = $this->is_valid_order($term_id, 'categories', $value,$coupon_amounts);
				if(is_array($rules_applied)){
					foreach($rules_applied as $rule){
						if($rule['applied_on']=='amount'){
							$wcol_min_value = wc_price( $rule['min-rule-limit'] );
							$wcol_max_value = $rule['max-rule-limit'] != '' ? wc_price( $rule['max-rule-limit'] ) : '';
						}else{
							$wcol_min_value = $rule['min-rule-limit'];
							$wcol_max_value = $rule['max-rule-limit'] != '' ? $rule['max-rule-limit'] : '';
						}
						$term = get_term( $term_id, 'product_cat');
						
						if($rule['enable-max-rule-limit']!='on'){
							$wcol_max_value = '<span style="font-size:30px; font-weight:bold;vertical-align: middle;">∞</span>';
						}
						if($rule['across-all-orders'] !='1'  and $rule['applied_for'] != 'minimum_limit'){
							if($rule['across-all-orders'] == '2'){
                            	$message = $wcol_settings['category_limit_message_across_all_orders'];
                            }else{
                            	$message = $wcol_settings['category_limit_message_across_all_users_orders'];
                            }
							switch($rule['rule-time-span']){
								case 'daily':
									$time_span_text = esc_html__('Daily', 'wc-order-limit');
									break;
								case 'weekly':
									$time_span_text = esc_html__('Weekly', 'wc-order-limit');
									break;
								case 'monthly':
									$time_span_text = esc_html__('Monthly', 'wc-order-limit');
									break;
								case 'yearly':
									$time_span_text = esc_html__('Yearly', 'wc-order-limit');
									break;	
								case 'custom':
									$time_span_text = esc_html__('from ', 'wc-order-limit').date('d M Y h:i a', $rule['from']). esc_html__(' to ', 'wc-order-limit').date('d M Y h:i a', $rule['to']);
									break;
							}
							$message = $wcol_settings['category_limit_message_across_all_orders'];
							$message = str_replace(
									array(
										'{category-name}',
										'{min-limit}',
										'{max-limit}',
										'{applied-on}',
										'{time-span}',
										'{limit-reset-day}',
										'{remaining}',
										'{endline}',
									),
									array(
										$term->name,
										$wcol_min_value,
										$wcol_max_value,
										$rule['applied_on'],
										$time_span_text,
										date($date_format, $rule['to']),
										$rule['remaining'],
										'<br>',
									),
									$message
								);
						}else{
							
							switch($rule['rule-time-span']){
								case 'daily':
									$time_span_text = esc_html__('Daily', 'wc-order-limit');
									break;
								case 'weekly':
									$time_span_text = esc_html__('Weekly', 'wc-order-limit');
									break;
								case 'monthly':
									$time_span_text = esc_html__('Monthly', 'wc-order-limit');
									break;
								case 'yearly':
									$time_span_text = esc_html__('Yearly', 'wc-order-limit');
									break;	
								case 'custom':
									$rule_start_time = date('d M Y h:i a', strtotime($rule['rule-start-time']));
									$rule_end_time = date('d M Y h:i a', strtotime($rule['rule-end-time']));
									$time_span_text = esc_html__('from ', 'wc-order-limit').date('d M Y h:i a', strtotime($rule['rule-start-time'])). esc_html__(' to ', 'wc-order-limit').date('d M Y h:i a', strtotime($rule['rule-end-time']));
									break;
							}
							$message = $wcol_settings['category_limit_message'];
							$time_span =  $this->wcol_get_time_span($rule, $this->get_wcol_settings());
							$date_format = apply_filters('xswcol_date_format','d M Y h:i:s a');
							$message = str_replace(
									array(
										'{category-name}',
										'{min-limit}',
										'{max-limit}',
										'{time-span}',
										'{applied-on}',
										'{limit-reset-day}',
										'{remaining}',
										'{endline}',
									),
									array(
										$term->name,
										$wcol_min_value,
										$wcol_max_value,
										$time_span_text,
										$rule['applied_on'],
										date('d M Y h:i:s a', $time_span['to']),
										$rule['remaining'],
										'<br>',

									),
									$message
								);
						}
						if( is_cart() ) {
							wc_print_notice(apply_filters('product_cat_order_limit_notice', $message, $term_id, $rule) , 'error' );
						} else {
							wc_add_notice( apply_filters('product_cat_order_limit_notice', $message, $term_id, $rule) , 'error' );
						}
						WC()->session->set('is_valid_order', false);
					}
				}
			}
		}
		/*	*	*	*	*	*	*	*	*	*	*	*	*/
		/*	 Accomulative Checks for Product Categories	*/
		/*	*	*	*	*	*	*	*	*	*	*	*	*/
		if(isset($wcol_settings['enable_category_limit']) && $wcol_settings['enable_category_limit']=='on' && is_array( $this->get_accomulative_rules_on_cart('categories',$coupon_amounts ) ) ){
			foreach($this->get_accomulative_rules_on_cart('categories',$coupon_amounts) as $rule){
				if( isset( $rule['applied_on'] ) && $rule[$rule['applied_on']] > 0 ){
					$cats = array();
					if( is_array($rule['obj_ids'] ) ){
						foreach($rule['obj_ids'] as $object_id){
							if($object_id != "-1" ){
								$term = get_term( $object_id, 'product_cat');
								$cats[] = $term->name;
							}else{
								$cats[] = "All Categories";
							}
						}
					}
					if($rule['applied_on']=='amount'){
						$wcol_min_value = wc_price( $rule['min-rule-limit'] );
						$wcol_max_value = $rule['max-rule-limit'] != '' ? wc_price( $rule['max-rule-limit'] ) : '';
					}else{
						$wcol_min_value = $rule['min-rule-limit'];
						$wcol_max_value = $rule['max-rule-limit'];
					}
					if(empty($wcol_max_value) || $rule['enable-max-rule-limit']!='on'){
						$wcol_max_value = '<span style="font-size:30px; font-weight:bold;vertical-align: middle;">∞</span>';
					}
					$message = $wcol_settings['category_limit_message_accomulative'];
					switch($rule['rule-time-span']){
						case 'daily':
							$time_span_text = esc_html__('Daily', 'wc-order-limit');
							break;
						case 'weekly':
							$time_span_text = esc_html__('Weekly', 'wc-order-limit');
							break;
						case 'monthly':
							$time_span_text = esc_html__('Monthly', 'wc-order-limit');
							break;
						case 'yearly':
							$time_span_text = esc_html__('Yearly', 'wc-order-limit');
							break;	
						case 'custom':
							$time_span_text = esc_html__('from ', 'wc-order-limit').date('d M Y h:i a', $rule['from']). esc_html__(' to ', 'wc-order-limit').date('d M Y h:i a', $rule['to']);
							break;
					}
					$message = str_replace(
							array(
								'{category-names}',
								'{min-limit}',
								'{max-limit}',
								'{applied-on}',
								'{time-span}',
								'{limit-reset-day}',
								'{remaining}',
								'{endline}'
							),
							array(
								implode(', ', $cats),
								$wcol_min_value,
								$wcol_max_value,
								$rule['applied_on'],
								$time_span_text,
								date($date_format, $rule['to']),
								$rule['remaining'],
								'<br>'
							),
							$message
						);
					if( is_cart() ) {
						wc_print_notice( apply_filters('accomulative_product_cats_limit_notice', $message, $rule), 'error' );
					} else {
						wc_add_notice( apply_filters('accomulative_product_cats_limit_notice', $message, $rule), 'error' );
					}
					WC()->session->set('is_valid_order', false);
				}
			}
		}
		if(!WC()->session->get('is_valid_order', false) && isset($wcol_settings['enable_checkout_button']) && $wcol_settings['enable_checkout_button']=='on'){
			remove_action( 'woocommerce_proceed_to_checkout',
			'woocommerce_button_proceed_to_checkout', 20);
		}
		date_default_timezone_set($default_time_zone);

	}
	
	public function get_wcol_option_by_user($object_id, $object_type){
		$wcol_options = $this->get_rules($object_id, $object_type);
		$matched_rules = array();
		if(is_array($wcol_options)){
			foreach($wcol_options as $key=>$option){
				if($option['disable-limit']!='on'){
					if(isset($option['enable-for-users']) && $option['enable-for-users'] == 'on'){
						switch($option['user-type']){
							case 'guest-users':
								if( !is_user_logged_in() ){
									$option['key']=$key;
									$matched_rules[] = $option;
								}
								break;
							case 'specific-users':
								if( is_user_logged_in() && in_array( get_current_user_id(), $option['rule-users'] ) ){
									$option['key']=$key;
									$matched_rules[] =  $option;
								}
								break;
							case 'specific-roles':
								$current_user = wp_get_current_user();
								$current_user_roles = $current_user->roles;
								if(is_array($current_user_roles)){
									foreach($current_user_roles as $role){
										if( is_user_logged_in() AND in_array( $role, $option['rule-roles'] ) ){
											$option['key']=$key;
											$matched_rules[] =  $option;
										}
									}
								}
								break;
							case 'all-users':
									$option['key']=$key;
									$matched_rules[] = $option;
								break;
							case 'membership':
								global $wpdb;
								$arr = implode("','", $option['rule-memberships']);
								$membership_query = "SELECT post_author FROM {$wpdb->prefix}posts
												WHERE post_status='wcm-active' AND post_type='wc_user_membership' AND post_parent IN ('".$arr."')
												ORDER BY ID";
								$memberships = $wpdb->get_results($membership_query);
								$user_ids = array();
								foreach($memberships as $user){
									$user_ids[] = $user->post_author;
								}
								if( is_user_logged_in() && in_array( get_current_user_id(),$user_ids) ){
									$option['key']=$key;
									$matched_rules[] =  $option;
								}
								break;
						}
					}else{
						$option['key']=$key;
						$matched_rules[] =  $option;
					}
				}
			}
		}
		
		if(empty($matched_rules)){
			return false;
		}
		return $matched_rules;
		
	}

	public function is_valid_order($object_id,$object_type,$total,$coupon_amounts,$cart_vid=null){
			date_default_timezone_set("UTC");
			$rules = $this->get_wcol_option_by_user($object_id, $object_type);
			
			if(!$rules)
				return true;
			$current_time = wp_date('y-m-d h:i:s A');
			$current_time = strtotime($current_time);
			$rules_applied = array();
			if (is_checkout()){
				$selected_payment_method_id = WC()->session->get( 'chosen_payment_method' );
			}else{
				$selected_payment_method_id ='';
			}
			
			if(is_array($rules)){
				foreach($rules as $rule){
					$rule['remaining'] ='';	
					$accomulative_check = (isset($rule['accomulative']) && $rule['accomulative']=='on') ? true : false;
					if($object_type == 'products'){
						$rule['max-rule-limit'] = $this->get_exclude_limit($object_id,$rule['max-rule-limit']);
						if($cart_vid && get_post_meta( $cart_vid, '_wcol_rule_limit', true ) == 'yes'){
							if(get_post_meta( $cart_vid, 'wcol_min_limit', true ) >= 0){
								$rule['min-rule-limit'] = get_post_meta( $cart_vid, 'wcol_min_limit', true );
							}
							if(get_post_meta( $cart_vid, 'wcol_max_limit', true ) > 0){
								$rule['max-rule-limit'] = get_post_meta( $cart_vid, 'wcol_max_limit', true );
							}
						}
					}
					
					if(!$accomulative_check){
						$rule['applied_for'] = '';
						if(isset($rule['across-all-orders']) && $rule['across-all-orders'] !='1' && $rule['enable-max-rule-limit']=='on' && $rule['max-rule-limit'] > 0 ){
							if($rule['across-all-orders'] == 'on' || $rule['across-all-orders'] == '2'){
								$across_all_orders_totals = $this->get_across_all_orders_totals($object_id, $rule,array(),$cart_vid);
							}else{
								$across_all_orders_totals = $this->get_across_all_user_orders_totals($object_id, $rule,array(),$cart_vid);
							}
							$cart_total = $total[$rule['applied_on']];			
							$gtotal = $total[$rule['applied_on']] + $across_all_orders_totals[$rule['applied_on']];
							$rule['from'] = $across_all_orders_totals['from'];
							$rule['to'] = $across_all_orders_totals['to'];
							$remain_limit = $rule['max-rule-limit'] - $across_all_orders_totals[$rule['applied_on']];
							if($remain_limit < 0){
								$remain_limit = 0;
							}

							$rule['remaining'] = $remain_limit;
							if($rule['applied_on'] == 'amount'){
								$cart_total = $cart_total+$coupon_amounts;
							}
							if($gtotal > $rule['max-rule-limit'] || $cart_total < $rule['min-rule-limit']){
								if($cart_total < $rule['min-rule-limit']){
									$rule['applied_for'] = 'minimum_limit';
									$rule['max-rule-limit'] = $rule['max-rule-limit'] - $across_all_orders_totals[$rule['applied_on']];
								}
								if(isset($rule['enable-for-payment']) && $rule['enable-for-payment'] == 'on'){
									if(isset($rule['payment-method']) && in_array($selected_payment_method_id,$rule['payment-method'])){
										$rules_applied[] = $rule;
									}
								}else{
									$rules_applied[] = $rule;
								}
								
							}
						}else{
							if(isset($rule['enable-time-limit']) && $rule['enable-time-limit'] == 'on' ){
								$wcol_settings = $this->get_wcol_settings();
								$time_span = $this->wcol_get_time_span($rule,$wcol_settings);
								$from = $time_span['from'];
								$to = $time_span['to'];
								if( $current_time < $from  || $current_time > $to){
									continue;
								}
								$rule['from'] = $from;
								$rule['to'] = $to;
							}

							$cart_total = $total[$rule['applied_on']];
							if($rule['applied_on'] == 'amount'){
								$cart_total = $total[$rule['applied_on']] + $coupon_amounts;
							}
							if($rule['enable-max-rule-limit']=='on' && ($cart_total < $rule['min-rule-limit'] || $cart_total > $rule['max-rule-limit']) ){
								$remain_limit = $rule['max-rule-limit'];
								if($remain_limit < 0){
									$remain_limit = 0;
								}
								$rule['remaining'] = $remain_limit;
								if(isset($rule['enable-for-payment']) && $rule['enable-for-payment'] == 'on'){
									if(isset($rule['payment-method']) && in_array($selected_payment_method_id,$rule['payment-method'])){
										$rules_applied[] = $rule;
									}
								}else{
									$rules_applied[] = $rule;
								}
							}elseif($total < $rule['min-rule-limit']){
								if(isset($rule['enable-for-payment']) && $rule['enable-for-payment'] == 'on'){
									if(isset($rule['payment-method']) && in_array($selected_payment_method_id,$rule['payment-method'])){
										$rules_applied[] = $rule;
									}
								}else{
									$rules_applied[] = $rule;
								}
							}
						}
					}
				}
			}
			if(empty($rules_applied)){
				return false;
			}
			
			return $rules_applied;
	}
		
	/*	get across all order orders for a gived rule */
	/*	@params	int, Array 							*/
	/*	returns	array								*/
	public function get_across_all_orders_totals($object_id, $rule,$orders_ids = array(),$cart_vid=null){
		$qty = 0;
		$amount = 0;
		$wcol_settings = $this->get_wcol_settings();
		$current_time = wp_date('y-m-d h:i:s A');
		
		$current_time = strtotime($current_time);
		$time_span = $this->wcol_get_time_span($rule,$wcol_settings);
		$from = $time_span['from'];
		$to = $time_span['to'];
		// Order Statuses Array 
		$order_statuses_custom = array('wc-pending','wc-processing','wc-on-hold','wc-completed');
		
		
		$args = array(
			'status' 			=> $order_statuses_custom,
			'order_type' 		=> wc_get_order_types(),
			'customer'			=> get_current_user_id(),
			'limit'				=>-1,
			'date_created'      => $from.'...'.$to,
		);
		if(!is_user_logged_in()){
			$args['customer']=0;
		}
		if(isset($rule["user-type"]) && $rule["user-type"]=="all-users"){
			unset($args['customer']);	
		}
		
		$_orders = wc_get_orders($args);
		
		$customer_orders = array();
		if(!is_user_logged_in()){
			if(is_array($_orders)){
				foreach($_orders as $order){
					$order_ip = get_post_meta($order->get_id(), '_customer_ip_address', true);
					if($_SERVER['REMOTE_ADDR'] == $order_ip){
						$customer_orders[] = $order;
					}
				}
			}
		}else{
			$customer_orders = $_orders;
		}
		$order_ids = array();
		if(is_array($customer_orders)){
			foreach($customer_orders as $order){
				if(is_array($order->get_items()) ){
					foreach($order->get_items() as $item ){
						switch($rule['rule-type']){
							case 'products':
								if(isset($rule['accomulative']) && $rule['accomulative'] == 'on'){
									if($rule['obj_ids'][0] == '-1'|| in_array($item->get_product_id() , $rule['obj_ids'] ) ){
										if(!in_array($order->get_id(), $orders_ids)){
											if($item->get_variation_id() > 0){
												$variation_id = $item->get_variation_id();
												if($cart_vid == $variation_id){
													$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product');
													$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product');
													$order_ids[$order->get_id()] = $order->get_id();
												} 
											}else{
												$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product');
												$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product');
												$order_ids[$order->get_id()] = $order->get_id();
											}
											
										}
									}
									
								}else{
									if($item->get_product_id() == $object_id){
										if($item->get_variation_id() > 0){
											$variation_id = $item->get_variation_id();
											if($cart_vid == $variation_id){
												$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product');
												$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product');
											} 
										}else{
											$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product');
											$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product');
										}
									}
								}
								break;
							case 'categories':
								if(isset($rule['accomulative']) && $rule['accomulative'] == 'on'){
									if($rule['obj_ids'][0] == '-1'){
										$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product_cat');
										$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product_cat');
									}elseif( has_term( $object_id, 'product_cat',  $item->get_product_id() ) ){
										$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product_cat');
										$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product_cat');
									}
								}else{
									if( has_term( $object_id, 'product_cat',  $item->get_product_id() ) ){
										$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product_cat');
										$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product_cat');
									}
								}
								break;
						}
					}
				} 
			}
		}
		$args = array(
					'quantity' 	=> $qty,
					'amount' 	=> $amount,
					'from'		=> $from,
					'to'		=> $to+1,
					'orders'    => $order_ids
				);
		$args = apply_filters( 'xswcol_get_across_all_orders_totals', $args , $customer_orders , $object_id , $rule );		
		return $args;
	}

	/*	get across all order orders for a gived rule */
	/*	@params	int, Array 							*/
	/*	returns	array								*/
	public function get_across_all_user_orders_totals($object_id, $rule,$orders_ids = array(),$cart_vid=null){
		$qty = 0;
		$amount = 0;
		$wcol_settings = $this->get_wcol_settings();
		$gmt_ofset = get_option('gmt_offset')*60*60;
		$current_time = wp_date('y-m-d h:i:s A');
		$current_time = strtotime($current_time);
		$time_span = $this->wcol_get_all_user_time_span($rule,$wcol_settings);
		$from = $time_span['from'];
		$to = $time_span['to'];
		
		// Order Statuses Array 
		$order_statuses_custom = array('wc-pending','wc-processing','wc-on-hold','wc-completed');
		
		
		$args = array(
			'status' 			=> $order_statuses_custom,
			'order_type' 		=> wc_get_order_types(),
			'limit'				=>-1,
			'date_created'      => $from.'...'.$to,
		);
		$_orders = wc_get_orders($args);
		$customer_orders = array();
		$customer_orders = $_orders;
		$order_ids = array();
		if(is_array($customer_orders)){
			foreach($customer_orders as $order){
				if(is_array($order->get_items()) ){
					foreach($order->get_items() as $item ){
						switch($rule['rule-type']){
							case 'products':
								if(isset($rule['accomulative']) && $rule['accomulative'] == 'on'){
									if($rule['obj_ids'][0] == '-1'|| in_array($item->get_product_id() , $rule['obj_ids'] ) ){
										if(!in_array($order->get_id(), $orders_ids)){
											if( $item->get_variation_id() > 0){
												$variation_id = $item->get_variation_id();
												if($cart_vid == $variation_id){
													$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product');
													$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product');
													$order_ids[$order->get_id()] = $order->get_id();
												} 
											}else{
												$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product');
												$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product');
												$order_ids[$order->get_id()] = $order->get_id();
											}
											
										}
									}
									
								}else{
									if($item->get_product_id() == $object_id){
										if($item->get_variation_id() > 0){
											$variation_id = $item->get_variation_id();
											if($cart_vid == $variation_id){
												$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product');
												$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product');
											} 
										}else{
											$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product');
											$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product');
										}
										
									}
								}
								break;
							case 'categories':
								if(isset($rule['accomulative']) && $rule['accomulative'] == 'on'){
									if($rule['obj_ids'][0] == '-1'){
										$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product_cat');
										$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product_cat');
									}elseif( has_term( $object_id, 'product_cat',  $item->get_product_id() ) ){
										$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product_cat');
										$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product_cat');
									}
								}else{
									if( has_term( $object_id, 'product_cat',  $item->get_product_id() ) ){
										$qty += apply_filters('wcol_order_item_qty', $item->get_quantity(), $item, 'product_cat');
										$amount += apply_filters('wcol_order_item_total', $item->get_subtotal(), $item, 'product_cat');
									}
								}
								break;
						}
					}
				} 
			}
		}
		$args = array(
			'quantity' 	=> $qty,
			'amount' 	=> $amount,
			'from'		=> $from,
			'to'		=> $to+1,
			'orders'    => $order_ids
		);
		$args = apply_filters( 'xswcol_get_across_all_users_orders_totals', $args , $customer_orders , $object_id , $rule );
		return $args;
	}
	public function get_wcol_settings(){
		$wcol_options = get_option('wcol_options',true);
        return $wcol_options['wcol_settings'];
	}

	public function wc_order_limit_restrict_checkout(){
		$wcol_settings = $this->get_wcol_settings();
		if(is_plugin_active('disable-cart-page-for-woocommerce/disable-cart-page-for-woocommerce.php')){
			return;
		}
		if(is_plugin_active('woocommerce-direct-checkout/woocommerce-direct-checkout.php')){
			return;
		}
		
		if(!WC()->cart->is_empty() && is_checkout() ){
			if(!WC()->session->get('is_valid_order')){
				if(isset($wcol_settings['enable_checkout_redirect']) && $wcol_settings['enable_checkout_redirect']=='on'){
					wp_redirect(wc_get_cart_url());
				}
				
			}
		}

	}

	/*	Save Order Creation Time Stamp */
	/*	@params	int Order ID			*/
	/*	returns	null					*/
	public function save_order_creation_timestamp($order_id){
		$time = strtotime(wp_date('y-m-d h:i:s A'));
		update_post_meta( $order_id, 'wcol_order_created', $time );
		if(is_user_logged_in()){
			$timespan = get_user_meta(get_current_user_id(),'xswcol_order_time_update',true);
			if(!$timespan){
				$timespan = 'no';
			}
			update_user_meta(get_current_user_id(),'xswcol_recent_order_time', $time);
			if(!get_user_meta(get_current_user_id(),'xswcol_order_time',true) || $timespan == 'no'){
				update_user_meta(get_current_user_id(),'xswcol_order_time', $time);
				update_user_meta(get_current_user_id(),'xswcol_order_time_update','yes');
			}
		}else{
			$timespan = get_option('xswcol_order_time_update_'.$_SERVER['REMOTE_ADDR'],true);
			update_option('xswcol_recent_order_time_'.$_SERVER['REMOTE_ADDR'],$time);
			if(!$timespan){
				$timespan = 'no';
			}
			if(!get_option('xswcol_order_time_'.$_SERVER['REMOTE_ADDR'],true) || $timespan == 'no'){
				update_option('xswcol_order_time_'.$_SERVER['REMOTE_ADDR'],$time);
				update_option('xswcol_order_time_update_'.$_SERVER['REMOTE_ADDR'],'yes');
			}
		}
	}

	public function wc_order_limit_footer_script(){
		$l10n = array();
		$script = '<script>jQuery(".woocommerce-cart-form__cart-item.cart_item .product-remove a.remove").on("click",function(){
				jQuery(".woocommerce-error").remove();
			});</script>';
		wp_localize_script( "wc-order-limit", $script,$l10n);
	}

	public function wcol_get_time_span($rule,$wcol_settings){
		$current_time = wp_date('y-m-d h:i:s A');
		$current_time = strtotime($current_time);
		if(is_user_logged_in()){
			$timespan = get_user_meta(get_current_user_id(),'xswcol_order_time',true);
			$recent_time =  get_user_meta(get_current_user_id(),'xswcol_recent_order_time', true);
			if($timespan){
				$from = intval($timespan);
			}else{
				$from  = $current_time;
			}
			if($recent_time){
				$recent_time = intval($recent_time);
			}else{
				$recent_time = $current_time;
			}
			switch($rule['rule-time-span']){
				case 'daily':
					if(isset($wcol_settings['reset_time']) && $wcol_settings['reset_time'] == 2) {
						$to = strtotime("+24 hours", $recent_time);
						if($to < $current_time){
							update_user_meta(get_current_user_id(),'xswcol_order_time_update','no');
							$from = $current_time;
							$to = strtotime("+24 hours", $from);
						}
					}else{
						$from = strtotime("midnight", $current_time);
						$to = strtotime("tomorrow", $from) - 1;
					}
					break;
				case 'weekly':
					if(isset($wcol_settings['reset_time']) && $wcol_settings['reset_time'] ==2) {
						$to = strtotime("+168 hours", $recent_time);
						if($to < $current_time){
							update_user_meta(get_current_user_id(),'xswcol_order_time_update','no');
							$from = $current_time;
							$to = strtotime("+168 hours", $from);
						}
					}else{
						$day =  isset($rule['weekly-start-day'])?$rule['weekly-start-day']:'monday';
						if(date('l') == ucfirst($day)){
							$from = strtotime('today UTC');
						}else{
							$day = "last ".$day;
							$from = strtotime($day);
						}
						$to = strtotime("+168 hours", $from);
					}
					break;
					
				case 'monthly':
					if(isset($wcol_settings['reset_time']) && $wcol_settings['reset_time'] ==2) {
						$to = strtotime("+1 month", $recent_time);
						if($to < $current_time){
							update_user_meta(get_current_user_id(),'xswcol_order_time_update','no');
							$from = $current_time;
							$to = strtotime("+1 month", $from);
						}
					}else{
						$year = date('Y');
						$month = date("m");
						$day = isset($rule['monthly-start-date'])?$rule['monthly-start-date']:'1';
						if($day > 0 && $day < 10){
							$day = $day;
						}
						$date = $year.'-'.$month.'-'.$day;
						$from  = strtotime($date);
						if($from > $current_time){
							$from = strtotime("-1 month", $from);
						}
						$to = strtotime("+1 month", $from);
					}
					break;
				
				case 'yearly':
					if(isset($wcol_settings['reset_time']) && $wcol_settings['reset_time'] ==2) {
						$to = strtotime("+1 year", $recent_time);
						if($to < $current_time){
							update_user_meta(get_current_user_id(),'xswcol_order_time_update','no');
							$from = $current_time;
							$to = strtotime("+1 year", $from);
						}
					}else{
						$year = date('Y');
						$month = isset($rule['yearly-start-month'])?$rule['yearly-start-month']:'01';
						$day = isset($rule['yearly-start-day'])?$rule['yearly-start-day']:'01';
						$date = $year.'-'.$month.'-'.$day;
						$from  = strtotime($date);
						if($from > $current_time){
							$from = strtotime("-1 year", $from);
						}
						$to = strtotime("+1 year", $from);
					}
					break;
					
				case 'custom':
					$from = strtotime($rule['rule-start-time']);
					$to = strtotime($rule['rule-end-time']);
					if( $current_time < $from || $current_time > $to){
						return array('quantity'=>-1, 'amount'=>-1,  'from'=>$from, 'to' => $to, 'amount' => '');
					}
					break;
			}
		}else{
			$timespan = get_option('xswcol_order_time_'.$_SERVER['REMOTE_ADDR'],true);
			$recent_time = get_option('xswcol_recent_order_time_'.$_SERVER['REMOTE_ADDR'],true);

			if($timespan){
				$from = intval($timespan);
			}else{
				$from  = $current_time;
			}

			if($recent_time){
				$recent_time = intval($recent_time);
			}else{
				$recent_time = $current_time;
			}
			switch($rule['rule-time-span']){
				case 'daily':
					if(isset($wcol_settings['reset_time']) && $wcol_settings['reset_time'] == 2) {
						$to = strtotime("+24 hours", $recent_time);
						if($to < $current_time){
							update_option('xswcol_order_time_update_'.$_SERVER['REMOTE_ADDR'],'no');
							$from = $current_time;
							$to = strtotime("+24 hours", $from);
						}
					}else{
						$from = strtotime("midnight", $current_time);
						$to = strtotime("tomorrow", $from) - 1;
					}
					break;
				case 'weekly':
					if(isset($wcol_settings['reset_time']) && $wcol_settings['reset_time'] ==2) {
						$to = strtotime("+168 hours", $recent_time);
						if($to < $current_time){
							update_option('xswcol_order_time_update_'.$_SERVER['REMOTE_ADDR'],'no');
							$from = $current_time;
							$to = strtotime("+168 hours", $from);
						}
					}else{
						$day =  isset($rule['weekly-start-day'])?$rule['weekly-start-day']:'monday';
						if(date('l') == ucfirst($day)){
							$from = strtotime('today UTC');
						}else{
							$day = "last ".$day;
							$from = strtotime($day);
						}
						$to = strtotime("+168 hours", $from);
					}
					break;
					
				case 'monthly':
					if(isset($wcol_settings['reset_time']) && $wcol_settings['reset_time'] ==2) {
						$to = strtotime("+1 month", $recent_time);
						if($to < $current_time){
							update_option('xswcol_order_time_update_'.$_SERVER['REMOTE_ADDR'],'no');
							$from = $current_time;
							$to = strtotime("+1 month", $from);
						}
					}else{
						$year = date('Y');
						$month = date("m");
						$day = isset($rule['monthly-start-date'])?$rule['monthly-start-date']:'1';
						if($day > 0 && $day < 10){
							$day = $day;
						}
						$date = $year.'-'.$month.'-'.$day;
						$from  = strtotime($date);
						if($from > $current_time){
							$from = strtotime("-1 month", $from);
						}
						$to = strtotime("+1 month", $from);
					}
					break;
				
				case 'yearly':
					if(isset($wcol_settings['reset_time']) && $wcol_settings['reset_time'] ==2) {
						$to = strtotime("+1 year", $recent_time);
						if($to < $current_time){
							update_option('xswcol_order_time_update_'.$_SERVER['REMOTE_ADDR'],'no');
							$from = $current_time;
							$to = strtotime("+1 year", $from);
						}
					}else{
						$year = date('Y');
						$month = isset($rule['yearly-start-month'])?$rule['yearly-start-month']:'01';
						$day = isset($rule['yearly-start-day'])?$rule['yearly-start-day']:'01';
						$date = $year.'-'.$month.'-'.$day;
						$from  = strtotime($date);
						if($from > $current_time){
							$from = strtotime("-1 year", $from);
						}
						$to = strtotime("+1 year", $from);
					}
					break;
					
				case 'custom':
					$from = strtotime($rule['rule-start-time']);
					$to = strtotime($rule['rule-end-time']);
					if( $current_time < $from || $current_time > $to){
						return array('quantity'=>-1, 'amount'=>-1,  'from'=>$from, 'to' => $to, 'amount' => '');
					}
					break;
			}
		}
		return array('from'=>$from, 'to' => $to);
	}

	public function wcol_get_all_user_time_span($rule,$wcol_settings){
		$current_time = wp_date('y-m-d');
		$current_time = strtotime($current_time);
		$from  = $current_time;
		switch($rule['rule-time-span']){
			case 'daily':
				$to = strtotime("+24 hours", $from);
				if($to < $current_time){
					$from = $current_time;
					$to = strtotime("+24 hours", $from);
				}
				break;
			case 'weekly':
				$day =  isset($rule['weekly-start-day'])?$rule['weekly-start-day']:'monday';
				if(date('l') == ucfirst($day)){
					$from = strtotime('today UTC');
				}else{
					$day = "last ".$day;
					$from = strtotime($day);
				}
				$to = strtotime("+168 hours", $from);
				break;
				
			case 'monthly':
				$year = date('Y');
				$month = date("m");
				$day = isset($rule['monthly-start-date'])?$rule['monthly-start-date']:'01';
				$date = $year.'-'.$month.'-'.$day;
				$from  = strtotime($date);
				if($from > $current_time){
					$from = strtotime("-1 month", $from);
				}
				$to = strtotime("+1 month", $from);
				break;
			
			case 'yearly':
				$year = date('Y');
				$month = isset($rule['yearly-start-month'])?$rule['yearly-start-month']:'01';
				$day = isset($rule['yearly-start-day'])?$rule['yearly-start-day']:'01';
				$date = $year.'-'.$month.'-'.$day;
				$from  = strtotime($date);
				if($from > $current_time){
					$from = strtotime("-1 year", $from);
				}
				$to = strtotime("+1 year", $from);
				break;
				
			case 'custom':
				$from = strtotime($rule['rule-start-time']);
				$to = strtotime($rule['rule-end-time']);
				if( $current_time < $from || $current_time > $to){
					return array('quantity'=>-1, 'amount'=>-1, 'from'=>$from, 'to' => $to, 'amount' => '');
				}
				break;
		}
		return array('from'=>$from, 'to' => $to);
	}

	/*	Get Accomulative rules applied in the cart 	*/
	/*	@params	string object Type								*/
	/*	returns	Array applied rules								*/
	public function get_accomulative_rules_on_cart($object_type,$coupon_amounts){
		$object_id = "accomulative";
		if (is_checkout()){
			$selected_payment_method_id = WC()->session->get( 'chosen_payment_method' );
		}else{
			$selected_payment_method_id ='';
		}
		date_default_timezone_set("UTC");
		$current_time = wp_date('y-m-d h:i:s A');
		$current_time = strtotime($current_time);
		$accomulative_rules = $this->get_wcol_option_by_user($object_id, $object_type);
		$applied_rules = array();
		if(is_array($accomulative_rules)){
			foreach($accomulative_rules as $rule){
				$qty=0;
				$amount=0;
				$other = 0;
				$limit  = 0;
				$product_items_arr = array();
				if(is_array($rule['obj_ids'])){
					foreach($rule['obj_ids'] as $id){
						foreach(WC()->cart->get_cart() as $key=>$cart_item){
							if( ($id==$cart_item['product_id'] || $id == "-1") && $object_type=='products'){
								if($this->check_exclude_rule($cart_item['product_id'])){
									continue;
								}
								$rule['max-rule-limit'] = $this->get_exclude_limit($cart_item['product_id'],$rule['max-rule-limit']);
								$product_items_arr[] = $cart_item;
							}elseif( (has_term( $id,'product_cat', $cart_item['product_id'] )|| $id == "-1" ) && $object_type=='categories'){
								$cart_item['category_id'] = $id;
								$product_items_arr[] = $cart_item;
							}else{
								continue;
							}
						}
					}
				}
				if(is_array($product_items_arr)){
					$orders = array();
					foreach ($product_items_arr as $key => $product_items) {
						$qty += apply_filters('wcol_cart_item_qty', $product_items['quantity'], $product_items, $object_type);
						$amount += apply_filters('wcol_cart_item_total', $product_items['line_total'], $product_items, $object_type);
						if(isset($rule['across-all-orders']) && $rule['across-all-orders'] != '1' && ($rule['enable-max-rule-limit']=='on' && $rule['max-rule-limit'] > 0) ){
							if( $object_type == 'products' ){								
								if($rule['across-all-orders'] == 'on' || $rule['across-all-orders'] == '2'){
									$across_all_orders_totals = $this->get_across_all_orders_totals($product_items['product_id'], $rule,$orders,$product_items['variation_id']);
								}else{
									$across_all_orders_totals = $this->get_across_all_user_orders_totals($product_items['product_id'], $rule,$orders,$product_items['variation_id']);
								}
								$orders = array_merge($orders,$across_all_orders_totals['orders']);
							}elseif($object_type == 'categories'){
								if($rule['across-all-orders'] == 'on' || $rule['across-all-orders'] == '2'){
									$across_all_orders_totals = $this->get_across_all_orders_totals($product_items['category_id'], $rule);
								}else{
									$across_all_orders_totals = $this->get_across_all_user_orders_totals($product_items['category_id'], $rule);
								}
							}
							if( $rule['applied_on'] == 'quantity' ){
								$qty = $qty + $across_all_orders_totals['quantity'];
							}elseif( $rule['applied_on'] == 'amount' ){
								$amount = $amount + $across_all_orders_totals['amount'];
							}else{
								$other = $across_all_orders_totals[$rule['applied_on']];
							}
							$limit = $limit + $across_all_orders_totals[$rule['applied_on']];
							$rule['from'] = $across_all_orders_totals['from'];
							$rule['to'] = $across_all_orders_totals['to'];
						}
						
					}
				}
				$rule['quantity'] = $qty;
				$rule['amount'] = $amount+$coupon_amounts;
				$remain_limit = $rule['max-rule-limit'] - $limit;
				if($remain_limit < 0){
					$remain_limit = 0;
				}
				$rule['remaining'] = $remain_limit;
				$rule = apply_filters('xswcol_get_accomulative_rules_on_cart' , $rule , $product_items_arr, $other);
				// check if rule time is not started yet or rule time is passed/
				if( $rule['enable-time-limit']=='on' && $rule['rule-start-time'] && $rule['rule-end-time'] ){
					$rule_start_time = strtotime($rule['rule-start-time']);
					$rule_end_time = strtotime($rule['rule-end-time']);
					if( $current_time < $rule_start_time || $current_time > $rule_end_time){
						//$applied_rules[]=$rule;
						continue;
					}
				}
				if(	$rule[$rule['applied_on']] < $rule['min-rule-limit'] || ($rule['enable-max-rule-limit'] == 'on' && $rule[$rule['applied_on']] > $rule['max-rule-limit']) ){
					if(isset($rule['enable-for-payment']) && $rule['enable-for-payment'] == 'on'){
						if(isset($rule['payment-method']) && in_array($selected_payment_method_id,$rule['payment-method'])){
							$applied_rules[] = $rule;
						}
					}else{
						$applied_rules[] = $rule;
					}
				}	
			}
		}
		return $applied_rules;
	}

	/*	Validate if Customer can place order */
	/*	@params	null						*/
	/*	returns	null						*/
	public function wcol_validate_order_for_customer(){
		$wcol_settings = $this->get_wcol_settings();
		if(!isset($wcol_settings['enable_customer_limit']) || $wcol_settings['enable_customer_limit'] != 'on'){
			return;
		}

		$rules_applied = $this->get_rule_applied_on_current_user();
		$default_time_zone = date_default_timezone_get();
		if($rules_applied == false){
			return;
		}

		if(is_array($rules_applied)){
			foreach($rules_applied as $rule_applied){	
				$total_orders = $this->get_total_orders($rule_applied);
				if($rule_applied['cus_applied_on'] == 'quantity'){
					$message = $wcol_settings['customer_message'];
					$rule_limit = $rule_applied['cus-max-rule-limit'];
				}else{
					$message = $wcol_settings['customer_message_total_amount'];
					$rule_limit = wc_price($rule_applied['cus-max-rule-limit']);
				}
				switch($rule_applied['rule-time-span']){
					case 'daily':
						$time_span_text = __('Daily', 'wc-order-limit');
						break;
					case 'weekly':
						$time_span_text = __('Weekly', 'wc-order-limit');
						break;
					case 'monthly':
						$time_span_text = __('Monthly', 'wc-order-limit');
						break;
					case 'yearly':
						$time_span_text = __('Yearly', 'wc-order-limit');
						break;
					case 'custom':
						$time_span_text = __('from ', 'wc-order-limit').date('d M Y h:i a', $total_orders['from']). __(' to ', 'wc-order-limit').date('d M Y h:i a', $total_orders['to']);
						break;
				}
				$date_format = apply_filters('xswcol_date_format','d M Y h:i:s a');
				$message = str_replace(
					array(
						'{rule-limit}',
						'{cus-applied-on}',
						'{time-span}',
						'{limit-reset-day}',
						'{remaining}'
					),
					array(
						$rule_limit,
						$rule_applied['cus_applied_on'],
						$time_span_text,
						date($date_format, $total_orders['to']+1),
						$total_orders['remaining']
					),
					$message
				);
				if($rule_applied){
					if($total_orders[$rule_applied['cus_applied_on']] && $total_orders[$rule_applied['cus_applied_on']] >  $rule_applied['cus-max-rule-limit']){
						if( is_cart() ) {
							wc_print_notice( apply_filters('customer_rule_limit_notice', $message, $rule_applied), 'error' );
						}else{
							wc_add_notice( apply_filters('customer_rule_limit_notice', $message, $rule_applied), 'error' );
						}
						WC()->session->set('is_valid_order', false);
					}
				}
			}
		}
		if(!WC()->session->get('is_valid_order', false) && isset($wcol_settings['enable_checkout_button']) && $wcol_settings['enable_checkout_button']=='on'){
			remove_action( 'woocommerce_proceed_to_checkout','woocommerce_button_proceed_to_checkout', 20);
		}
		date_default_timezone_set($default_time_zone);
	}
	/*	get rule applied on current user 	*/
	/*	@params	null						*/
	/*	returns	null						*/
	public function get_rule_applied_on_current_user(){
		$customer_rules = $this->get_wcol_customer_rules();
		if (is_checkout()){
			$selected_payment_method_id = WC()->session->get( 'chosen_payment_method' );
		}else{
			$selected_payment_method_id ='';
		}
		if(!is_array($customer_rules) || empty($customer_rules))
			return false;
		$rules_applied = array();
		if(is_array($customer_rules)){
			foreach($customer_rules as $rule){
				if($rule['disable-limit'] != 'on'){
					if(in_array('all-users',$rule['obj_ids']) || in_array('every-user',$rule['obj_ids'])){
						$rules_applied[] = $rule;
					}elseif( in_array('selective-users' , $rule['obj_ids']) && is_user_logged_in()){
						if(in_array(get_current_user_id(), $rule['cus-rule-users'])){
							if(isset($rule['enable-for-payment']) && $rule['enable-for-payment'] == 'on'){
								if(isset($rule['payment-method']) && in_array($selected_payment_method_id,$rule['payment-method'])){
									$rules_applied[] = $rule;
								}
							}else{
								$rules_applied[] = $rule;
							}
						}
					}elseif(in_array('roles',$rule['obj_ids']) && is_user_logged_in()){
						$current_user = wp_get_current_user();
						$current_user_roles = $current_user->roles;
						if(is_array($current_user_roles)){
							foreach($current_user_roles as $role){
								if(!empty($rule['cus-rule-roles']) && in_array($role, $rule['cus-rule-roles'])){
									if(isset($rule['enable-for-payment']) && $rule['enable-for-payment'] == 'on'){
										if(isset($rule['payment-method']) && in_array($selected_payment_method_id,$rule['payment-method'])){
											$rules_applied[] = $rule;
										}
									}else{
										$rules_applied[] = $rule;
									}
								}
							}
						}
					}elseif(in_array('guest-users',$rule['obj_ids']) && !is_user_logged_in()){
						if(isset($rule['enable-for-payment']) && $rule['enable-for-payment'] == 'on'){
							if(isset($rule['payment-method']) && in_array($selected_payment_method_id,$rule['payment-method'])){
								$rules_applied[] = $rule;
							}
						}else{
							$rules_applied[] = $rule;
						}
					}
				}
			}
		}
		if(count($rules_applied) > 0 ){
			return $rules_applied;
		}
		return false;
	}

	/*	GET wcol Customer Rules	*/
	/*	@params	null			*/
	/*	returns	Array			*/
	public function get_wcol_customer_rules(){
		$args = array(
			'fields' => 'ids',
		    'post_type'=> 'wcol_rule',
		    'status'    => 'publish',
		    'numberposts' => -1,
		);              
		$rule_ids = get_posts($args);
		$match_rules = array();
		foreach ($rule_ids as $id) {
			$rules = get_post_meta($id,'wcol-post-rules',true);
			if($rules['rule-type'] == 'customer'){
				$match_rules[] = $rules;

			} 

		}
		if(empty($match_rules)){
			return false;
		}else{
			return $match_rules;
		}
	}
	/*	get total orders				  	*/
	/*	@params	array						*/
	/*	returns	array						*/
	public function get_total_orders($rule){
		global $woocommerce;
		date_default_timezone_set("UTC");
		$wcol_settings = $this->get_wcol_settings();
		$current_time = wp_date('y-m-d h:i:s A');
		$current_time = strtotime($current_time);
		$time_span = $this->wcol_get_time_span($rule,$wcol_settings);
		$from = $time_span['from'];
		$to = $time_span['to'];
		
		// Order Statuses Array 
		$order_statuses_custom = array('wc-pending','wc-processing','wc-on-hold','wc-completed');
		
		
		$args = array(
			'status' 			=> $order_statuses_custom,
			'order_type' 		=> wc_get_order_types(),
			'customer'			=> get_current_user_id(),
			'limit'				=>-1,
			'date_created'      => $from.'...'.$to,
		);
		
		if(!is_user_logged_in()){
			$args['customer']=0;
		}
		if(in_array('all-users',$rule['obj_ids'])){
            unset($args['customer']);
		}
		$_orders = wc_get_orders($args);
		$customer_orders = array();
		if(!is_user_logged_in()){
			if(is_array($_orders )){
				foreach($_orders as $order){
					$order_ip = get_post_meta($order->get_id(), '_customer_ip_address', true);
					if($_SERVER['REMOTE_ADDR'] == $order_ip){
						$customer_orders[] = $order;
					}
				}
			}
		}else{
			$customer_orders = $_orders;
		}
		if(in_array('all-users',$rule['obj_ids'])){
            $customer_orders = $_orders;
		}
		$wcol_orders_amount = $woocommerce->cart->total;
		$wcol_orders_quantity=1;
		if(is_array($customer_orders)){
			foreach($customer_orders as $order){
				$wcol_orders_amount+=apply_filters('wcol_order_total_amount',$order->get_total(), $order);
				$wcol_orders_quantity++;
			}
		}
		$remain_limit = 0;
		if($rule['cus_applied_on'] == 'quantity'){
			$remain_limit = $rule['cus-max-rule-limit'] - $wcol_orders_quantity;
		}else{
			$remain_limit = $rule['cus-max-rule-limit'] - ($wcol_orders_amount-$woocommerce->cart->total);
		}
		if($remain_limit < 0){
			$remain_limit = 0;
		}
		$args = array(
					'quantity' 	=> $wcol_orders_quantity,
					'amount' 	=> $wcol_orders_amount,
					'from'		=> $from,
					'to'		=> $to,
					'remaining' => $remain_limit
				);
		$args = apply_filters( 'xswcol_get_customer_orders_totals', $args , $customer_orders , $rule );		
		return $args;
	}
	/*	save wcol options	*/
	/*	@params	int,string	*/
	/*	returns	null		*/
	
	public function save_wcol_options($object_id, $object_type){
		$raw_rules = isset($_POST['wcol_rules']) ? $_POST['wcol_rules'] : array();
		$wcol_options = array();
		if(count($raw_rules) > 0){
			if(is_array($raw_rules['min-rule-limit'])){
				foreach($raw_rules['min-rule-limit'] as $key=>$value){
					$rule_id = $raw_rules['id'][$key];
					$rules = get_post_meta($rule_id,'wcol-post-rules',true);
					if($rules){
						$rules['min-rule-limit'] = stripcslashes( $raw_rules['min-rule-limit'][$key] )!='' ? stripcslashes( $raw_rules['min-rule-limit'][$key] ) : 0;
						$rules['applied_on'] = stripcslashes( $raw_rules['applied_on'][$key]);
						$rules['disable-limit'] = stripcslashes( $raw_rules['disable-limit'][$key]);
						$rules['enable-max-rule-limit'] = stripcslashes( $raw_rules['enable-max-rule-limit'][$key]);
						$rules['max-rule-limit'] = stripcslashes( $raw_rules['max-rule-limit'][$key]);
						$rules['enable-time-limit'] = stripcslashes( $raw_rules['enable-time-limit'][$key]);
						$rules['rule-time-span'] = stripcslashes( $raw_rules['rule-time-span'][$key]);
						$rules['yearly-start-day'] = stripcslashes( $raw_rules['yearly-start-day'][$key]);
						$rules['yearly-start-month'] = stripcslashes( $raw_rules['yearly-start-month'][$key]);
						$rules['weekly-start-day'] = stripcslashes( $raw_rules['weekly-start-day'][$key]);
						$rules['monthly-start-date'] = stripcslashes( $raw_rules['monthly-start-date'][$key]);
						$rules['enable-for-users'] = stripcslashes( $raw_rules['enable-for-users'][$key]);
						$rules['rule-start-time'] = stripcslashes( $raw_rules['rule-start-time'][$key]);
						$rules['rule-end-time'] = stripcslashes( $raw_rules['rule-end-time'][$key]);
						if($object_type != 'vendor'){
							$rules['across-all-orders'] = stripcslashes( $raw_rules['across-all-orders'][$key]);
							$rules['accomulative'] = stripcslashes( $raw_rules['accomulative'][$key]);
							$rules['user-type'] = stripcslashes( $raw_rules['user-type'][$key]);
							if(isset($raw_rules['rule-users'][$key])){
								$rules['rule-users'] = $raw_rules['rule-users'][$key];
							}
							if(isset($raw_rules['rule-roles'][$key])){
								$rules['rule-roles'] = $raw_rules['rule-roles'][$key];
							}

						}else{
							$rules['editable'] = $raw_rules['editable'][$key];
						}
						update_post_meta($rule_id,'wcol-post-rules',$rules);
					}else{
						$rules = array();
						$rules['rule-type'] =  stripcslashes($object_type);
						$rules['obj_ids'] = array($object_id);
						$rules['min-rule-limit'] = stripcslashes($raw_rules['min-rule-limit'][$key] )!='' ? stripcslashes( $raw_rules['min-rule-limit'][$key] ) : 0 ;
						$rules['applied_on'] = stripcslashes($raw_rules['applied_on'][$key]);
						$rules['disable-limit'] = isset($raw_rules['disable-limit'][$key] ) ? stripcslashes( $raw_rules['disable-limit'][$key] ) : '' ;
						$rules['enable-max-rule-limit'] = stripcslashes( $raw_rules['enable-max-rule-limit'][$key]);
						$rules['max-rule-limit'] = stripcslashes( $raw_rules['max-rule-limit'][$key]);
						$rules['enable-time-limit'] = stripcslashes( $raw_rules['enable-time-limit'][$key]);
						$rules['rule-time-span'] = stripcslashes( $raw_rules['rule-time-span'][$key]);
						$rules['yearly-start-day'] = stripcslashes( $raw_rules['yearly-start-day'][$key]);
						$rules['yearly-start-month'] = stripcslashes( $raw_rules['yearly-start-month'][$key]);
						$rules['weekly-start-day'] = stripcslashes( $raw_rules['weekly-start-day'][$key]);
						$rules['monthly-start-date'] = stripcslashes( $raw_rules['monthly-start-date'][$key]);
						$rules['enable-for-users'] = stripcslashes( $raw_rules['enable-for-users'][$key]);
						$rules['rule-start-time'] = stripcslashes( $raw_rules['rule-start-time'][$key]);
						$rules['rule-end-time'] = stripcslashes( $raw_rules['rule-end-time'][$key]);
						if($object_type != 'vendor'){
							$rules['accomulative'] = isset($raw_rules['accomulative'][$key] ) ? stripcslashes( $raw_rules['accomulative'][$key] ) : '' ;
							$rules['across-all-orders'] = stripcslashes( $raw_rules['across-all-orders'][$key]);
							$rules['user-type'] = stripcslashes( $raw_rules['user-type'][$key]);
							if(isset($raw_rules['rule-users'][$key])){
								$rules['rule-users'] = $raw_rules['rule-users'][$key];
							}
							if(isset($raw_rules['rule-roles'][$key])){
								$rules['rule-roles'] = $raw_rules['rule-roles'][$key];
							}
						}else{
								$rules['editable'] = $raw_rules['editable'][$key];
						}
						$post_arr = array(
						    'post_title'   => get_the_title($object_id).' rule',
						    'post_type'    => 'wcol_rule',
						    'post_status'  => 'publish',
						    'post_author'  => get_current_user_id(),
						);
						if($object_type == 'categories'){
							$term = get_term( $object_id, 'product_cat' );
							$post_arr['post_title'] = $term->name.' rule';
						}
						if($object_type == 'vendor'){
							$user = get_user_by('id', $object_id);
							$post_arr['post_title'] = $user->display_name.' vendor rule';
						}
						$rule_id = wp_insert_post($post_arr);
						if($rule_id){
							update_post_meta($rule_id,'wcol-post-rules',$rules);
						}
					}
				}
			}
		}
	}
	/*	Get Product Base Rules	*/
	/*	@params	null			*/
	/*	returns	Array			*/
	public function get_exclude_rules(){
		$wcol_options = get_option('wcol_options');
		if(isset($wcol_options['wcol_exclude_rules'])){
			return $wcol_options['wcol_exclude_rules'];
		}else{
			return array();
		}
		
	}
	public function get_exclude_limit($id,$limit){
    	$wcol_exculde_rules = $this->get_exclude_rules();
    	if(is_array($wcol_exculde_rules)){
    		foreach ($wcol_exculde_rules as $rule ) {
    			if(in_array($id, $rule['object_ids']) || in_array('-1', $rule['object_ids']) ){
    				if(!isset($rule['disable-limit']) ||  $rule['disable-limit'] !='on'){
    					if(isset($rule['wcol_max_order_limit']) && $rule['wcol_max_order_limit'] > 0 ){
    						$limit = $rule['wcol_max_order_limit'];
    						return $limit;
    					}
    				}
    			}
    		}
    	}
    	return $limit;
    }
	public function check_exclude_rule($id){
    	$wcol_exculde_rules = $this->get_exclude_rules();
    	if(is_array($wcol_exculde_rules)){
    		foreach ($wcol_exculde_rules as $rule ) {
    			if(in_array($id, $rule['object_ids']) || in_array('-1', $rule['object_ids']) ){
    				if(!isset($rule['disable-limit']) ||  $rule['disable-limit'] !='on'){
    					if(isset($rule['exclude-limit']) && $rule['exclude-limit'] == 'on' ){
    						return true;
    					}
    				}
    			}
    		}
    	}
    	return false;
    }
	
	public function valid_store_limit($wcol_settings){
		$current_time = wp_date('y-m-d h:i:s A');
		$current_time = strtotime($current_time);
		if(isset($wcol_settings['wcol_store_time_span'])){
			switch($wcol_settings['wcol_store_time_span']){
				case 'daily':
					$from = strtotime("midnight", $current_time);
					$to = strtotime("tomorrow", $from) - 1;
					break;
				case 'weekly':
					$day =  isset($wcol_settings['wcol_store_weekly_start_day'])?$wcol_settings['wcol_store_weekly_start_day']:'monday';
					if(date('l') == ucfirst($day)){
						$from = strtotime('today UTC');
						
					}else{
						$day = "last ".$day;
						$from = strtotime($day);
					}
					$to = strtotime("+168 hours", $from);
					break;
					
				case 'monthly':
					$year = date('Y');
					$month = date("m");
					$day = isset($wcol_settings['wcol_store_monthly_start_date'])?$wcol_settings['wcol_store_monthly_start_date']:'01';
					$date = $year.'-'.$month.'-'.$day;
					$from  = strtotime($date);
					if($from > $current_time){
						$from = strtotime("-1 month", $from);
					}
					$to = strtotime("+1 month", $from);
					break;
				
				case 'yearly':
					$year = date('Y');
					$month = isset($wcol_settings['wcol_store_yearly_start_month'])?$wcol_settings['wcol_store_yearly_start_month']:'01';
					$day = isset($wcol_settings['wcol_store_yearly_start_day'])?$wcol_settings['wcol_store_yearly_start_day']:'01';
					$date = $year.'-'.$month.'-'.$day;
					$from  = strtotime($date);
					if($from > $current_time){
						$from = strtotime("-1 year", $from);
					}
					$to = strtotime("+1 year", $from);
					break;
					
				case 'custom':
					$from = strtotime($wcol_settings['wcol_store_start_time']);
					$to = strtotime($wcol_settings['wcol_store_end_time']);
					break;
			}
			$order_statuses_custom = array('wc-pending','wc-processing','wc-on-hold','wc-completed');
			$args = array(
				'status' 			=> $order_statuses_custom,
				'order_type' 		=> wc_get_order_types(),
				'limit'				=>-1,
				'date_created'      => $from.'...'.$to,
			);
			$orders = wc_get_orders($args);
			$total_orders = count($orders);
			$total_amount = WC()->cart->total;
			foreach($orders as $order){
				$total_amount = $total_amount+$order->get_total();
			}
			if(isset($wcol_settings['store_max_limit']) && $total_orders > $wcol_settings['store_max_limit'] || isset($wcol_settings['store_max_amount_limit']) && $total_amount > $wcol_settings['store_max_amount_limit'] ){
				return array('from'=>$from,'to'=>$to);
			}
		}
		return false;
	}
		
}